#!/bin/awk -f
## Filters MASS file background data
## 
## Parameters: MAX_BG_D - maximal background in D-aperture
##             FIXED_BG_D - background to set if no valid bkgr data found
##             RAT_F_AD, RAT_F_BD, RAT_F_CD - ratios of fluxes
## Parameters are taken from command line or defaults are used
##
## Description: find "bad" backgrounds: > maximal, computed from MAX_BG_D and 
## RAT_F_AD, RAT_F_BD, RAT_F_CD. 
## If "bad" after at least one "good" before - remove the measurement. 
## If "bad" with no one "good" before - replace with FIXED_BG_D*RAT_F_*.
##
## Use as: bkgr.awk [<]<mass-file> [MAX_BG_D=val1] [FIXED_BG_D=val2] [RAT_F_AD|BD|CD=val3|4|5] >result-file
BEGIN{
	# Default threshold and guess-background:
	MAX_BG_D=3 
	FIXED_BG_D=0.5
	
	# Default ratios of fluxes are invalid!!
	RAT_F_AD=0
	RAT_F_BD=0
	RAT_F_CD=0

	# Make field names search more robust to future soft evolution:
	IGNORECASE = 1	
	# Field numbers: 
	n_P = 4
	n_D = 8
	nreject = 0
	ngood = 0
}

NR==1 {
	print "## ==== BKGR REMOVE IF BKGR_D>"MAX_BG_D" or REPLACE WITH "FIXED_BG_D
	if (RAT_F_AD * RAT_F_BD * RAT_F_CD <=0)
		print "## WARNING! FLUX RATIOS ARE BAD! AD="RAT_F_AD" BD="RAT_F_BD" CD="RAT_F_CD
}

$1=="P" && $0~/.*Background=.*/ {
	if ($(n_D)>MAX_BG_D) {
		nreject++
		if (ngood==0) {
			# output warning on STDERR:
			print $2"T"$3" BKGR DEFAULTS USED"  > "/dev/stderr"
			print "## BKGR REPLACE "$0
			gsub("=.*","="FIXED_BG_D*RAT_F_AD", "FIXED_BG_D*RAT_F_BD", "FIXED_BG_D*RAT_F_CD", "FIXED_BG_D)
			ngood++
		} else {
			print "## BKGR REJECT "$0
			getline
		}
	} else ngood++
}

!($1=="##" && $0~/BKGR/) { print }

END{
    print "## BKGR NREJECTED="nreject
}
