#!/bin/sh
#######################################################################
## Main script for MASS data reprocessing
## Parameter: the base-name(s) of the mass- and stm-file YYMMDD.
## 
## Bulk processing may be organized as follows:
## ./remass.sh ../in/*.mass 2>remass.log

############ Program and script path settings #########################
############ (post-installation tuning only) ##########################
BKGR=./bkgr.awk
PARAM=./param.awk
DTIME=./dtime.awk
FILTER=./filter.awk
TURBDIR="/opt/turbina/data"
WEIGHTDIR="./weight"
SPECDIR=$TURBDIR/"spectra"
RESPFILE=$TURBDIR/"data/mass.crv"
ATMOS="/opt/turbina/bin/atmos"
ATMOSCALL=$ATMOS" -r"$RESPFILE" -w"$WEIGHTDIR" -s"$SPECDIR
OUT=./out
################# Reprocessing parameters #############################
########### (Modify whenever any condition changes) ###################

# This setting regulates wrong background cleaning (RAT_F_* are OBLIGATORY!!!)
# Example: BKGR_DATA="FIXED_BG_D=1 MAX_BG_D=5 RAT_F_AB=0.37 RAT_F_BC=0.24 RAT_F_CD=0.30"
BKGR_DATA="

"

# Here are TRUE optics magnification, PMT characteristics and scattering (if any)
# Example: DEVICE_DATA="k=15.7 PA=1.01 PB=1.016 nA=22 nD=23 SB=0.05"
DEVICE_DATA="

"

# Here we cause implausible results to be commented out
FILTER_OUT_DATA="
MAX_BG_F_D=5 MAX_CHI2=100 MIN_D=100 MAX_E_D=0.02 MAX_E_FSEE=2.0
"

# Here we set dt=3600 or dt=-3600 ONLY for data with +/- 1 hour wrong UT reference
DTIME_DATA="
dt=0
"

################### Making reprocessing ###############################
############## (do not change anything here) ##########################
ok=1
for x in $BKGR $PARAM $DTIME $FILTER $ATMOS; do
	if [ ! -x $x ]; then
		echo "No program $x found or wrong permissions"
		echo "Make sure you start remass.sh from its directory."
		ok=0
	fi
done
for d in $WEIGHTDIR $SPECDIR $IN $OUT; do
	if [ ! -d $d ]; then
		echo "No directory $d found"
		ok=0
	fi
done
for f in $RESPFILE; do
	if [ ! -f $f ]; then
		echo "No file $f found"
		ok=0
	fi
done
if [ $ok -ne 1 ]; then exit 1; fi

for mass in `ls $@`; do
	i=`echo $mass|sed -e s/mass$//`
	o=`echo $i|sed -e sx^.*/xxg`
	stm=$i"stm"
	dt=`echo $DTIME_DATA|sed s/dt=//`
	if [ $dt != 0 -a -n $dt ]; then
		ostm=$OUT/$o"r.stm"
		echo "process $stm in $ostm"
		$DTIME $DTIME_DATA $stm >$ostm
		stm=$ostm
	fi
	omass=$OUT/$o"r.mass"
	echo "process $mass with $stm in $omass"
	$BKGR $BKGR_DATA $mass |\
	$PARAM $DEVICE_DATA |\
	$DTIME $DTIME_DATA |\
	$ATMOSCALL - $stm |\
	$FILTER $FILTER_OUT_DATA >$omass
done
#######################################################################
