#include <stdlib.h>	// To prevent problems with missing built-in abs():
#include <math.h>
#include "bicounter.h"
#include "tools.h"
#include "exchange.h"
#include "cfg.h"

Bicounter::Bicounter(void) {
	jdebug("bicounter::bicounter"); 
	_name = "Bicounters";
	resetVariables();
	jdebug("end of Bicounter's constructor");
}

Bicounter::~Bicounter() {
	jdebug("Destructor of Bicounter\n");
	delete[] dataA;
	delete[] dataB;
	jdebug("End of Destructor of Bicounter\n");
}

void Bicounter::resetVariables(void) {
	jdebug("bicounter::resetVariables");
	ProCounter::resetVariables();
	_levelValueA = UNDEFINED;
	_levelValueB = UNDEFINED;
	jdebug("End of bicounter's resetVariables");
}

void Bicounter::create(int sizeOfData) {
	ProCounter::create(sizeOfData);
	_nDataA = 0;
	_nDataB = 0;
	_expanded = false;
	dataA = new count_t[_maxNData];
	dataB = new count_t[_maxNData];
}

int Bicounter::loadOwnConst(void) {
	jdebug("bicounter::loadOwnConst");
	bool first = true;
	leafDataToChar(_setLevelA,first,deviceCfgFile,_name,"Command","SetLevelA");
	leafDataToChar(_setLevelB,first,deviceCfgFile,_name,"Command","SetLevelB");
	leafDataToChar(_getLevelA,first,deviceCfgFile,_name,"Command","GetLevelA");
	leafDataToChar(_getLevelB,first,deviceCfgFile,_name,"Command","GetLevelB");

	leafDataToDouble(_workLevelA,first,deviceCfgFile,_name,_identName,"DiscriminationA");
	leafDataToDouble(_workLevelB,first,deviceCfgFile,_name,_identName,"DiscriminationB");
	leafDataToDouble(_nonLinearityA,first,deviceCfgFile,_name,_identName,"NonLinearityA");
	leafDataToDouble(_nonLinearityB,first,deviceCfgFile,_name,_identName,"NonLinearityB");
	leafDataToDouble(_nonPoissonA,first,deviceCfgFile,_name,_identName,"NonPoissonA");
	leafDataToDouble(_nonPoissonB,first,deviceCfgFile,_name,_identName,"NonPoissonB");

	jdebug("End of bicounter's loadConst");
	if(_setLevelA==0) return FAIL;
	return SUCCESS;
}

int Bicounter::loadConst(void) {
	jdebug("bicounter::loadConst");
	if(HardModule::loadOwnConst() == FAIL) return FAIL;
	if(ProCounter::loadOwnConst() == FAIL) return FAIL;
	return loadOwnConst();
}

void Bicounter::resizeData(int newSize) {
	if(newSize > _maxNData) {
		_maxNData = newSize;
		delete[] dataA;
		delete[] dataB;
		dataA = new count_t[_maxNData];
		dataB = new count_t[_maxNData];
		_expanded = false;
	}
}

bool Bicounter::expandData(void) {
	if(_expanded) return _isCorrected;
	_expanded = true;
	int nCountInBlock = _blockSize / BYTE_PER_COUNT;

	_isCorrected = false;

	if(!nCountInBlock) return _isCorrected;
	int finiteLabel = _expositionNumber / nCountInBlock * 2;

	for(int label=0; label<finiteLabel; label++) {
		if(label >= _nBlockLabel) {
			insertBlock(label, INVALID_COUNT);
			_isCorrected = true;
			continue;
		}
		if(blockLabel[label] > label) { // skipped
			insertBlock(label,INVALID_COUNT);
			_isCorrected = true;
			continue;
		}
		if(blockLabel[label] < label) { // multiplieds
			if(removeBlock(label) == 0) break;
			label--;
			continue;
		}
	}
	return _isCorrected;
}

int Bicounter::repairData(char counter) {
	double avg_d = 0;
	int nValid = 0 ;
	count_t *data;
	int* nData;
	if(counter == 'A') {
		data = dataA;
		nData = &_nDataA;
	} else {
		data = dataB;
		nData = &_nDataB;
	}
	for(int i=0; i<*nData; i++) {
		if(data[i] != INVALID_COUNT) {
			avg_d += data[i];
			nValid++;
		}
	}
	if(nValid == 0) {
		// bad... nothing to divide for: return number of "corrected" = nData
		return *nData ;
	} 
	count_t avg = (count_t)rint(avg_d / nValid);

	// Replace INVALIDs with average:
	int nCorrected = 0;
	for(int i=0; i<*nData; i++) {
		if(data[i] == INVALID_COUNT) {
			data[i] = avg;
			nCorrected++;
		}
	}
	return nCorrected ;
}

const int N_COUNTERS_IN_BICOUNTER = 2;

int Bicounter::insertBlock(int label, count_t value) {
	if(_nDataA != _nDataB) {
		jdebug("_nDataA=%d != _nDataB=%d !!!\n",_nDataA,_nDataB);
		return 0;
	}
	int nData = _nDataA;

	int _nCountInBlock = _blockSize / BYTE_PER_COUNT;
	int _nCountInBlockPerCounter = _nCountInBlock / N_COUNTERS_IN_BICOUNTER;

	int start = 0;
	int startLabel = 0;
	for(int i=0; i<_nBlockLabel; i++) {
		if(blockLabel[i] == label-1) {
			start = (i+1)*_nCountInBlockPerCounter;
			startLabel = i+1;
		}
	}
	if((nData+_nCountInBlockPerCounter) > _maxNData) {
		jdebug("nData+_nCountInBlockPerCounter = %d > _maxNData = %d",nData+_nCountInBlockPerCounter,_maxNData);
		return(0);
	}
	nData += _nCountInBlockPerCounter;
	for(int j=(nData-1); j>=(start+_nCountInBlockPerCounter); j--) {
		dataA[j] = dataA[j-_nCountInBlockPerCounter];
		dataB[j] = dataB[j-_nCountInBlockPerCounter];
	}
	for(int j=start; j<(start+_nCountInBlockPerCounter); j++) {
		dataA[j] = value;
		dataB[j] = value;
	}
	_nDataA = nData;
	_nDataB = nData;
	_nBlockLabel++;
	for(int j=_nBlockLabel; j>startLabel; j--) {
		blockLabel[j] = blockLabel[j-1];
	}
	blockLabel[startLabel] = label;
	jdebug("%s: block %d inserted",_identName,label);
	return(1);
}

int Bicounter::removeBlock(int label) {
	if(_nDataA != _nDataB) {
		jdebug("_nDataA=%d != _nDataB=%d !!!\n",_nDataA,_nDataB);
		return 0;
	}
	int nData = _nDataA;

	int _nCountInBlock = _blockSize / BYTE_PER_COUNT;
	int _nCountInBlockPerCounter = _nCountInBlock / N_COUNTERS_IN_BICOUNTER;
	int start;

	for(int i=0; i<_nBlockLabel; i++) {
		if(blockLabel[i] == label) {
			start = i*_nCountInBlockPerCounter;
			if(start+_nCountInBlockPerCounter >= nData) {
				jdebug("start+_nCountInBlockPerCounter = %d > nData = %d",start+_nCountInBlockPerCounter, nData);
				return(0); // not found
			}
			nData -= _nCountInBlockPerCounter;
			for(int j=start; j<nData; j++) {
				dataA[j] = dataA[j+_nCountInBlockPerCounter];
				dataB[j] = dataB[j+_nCountInBlockPerCounter];
			}
			_nDataA = nData;
			_nDataB = nData;
			_nBlockLabel--;
			for(int j=i; j<_nBlockLabel; j++) {
				blockLabel[j] = blockLabel[j+1];
			}
			jdebug("%s: block %d removed",_identName,label);
			return(1);
		}
	}
	jdebug("block with label %d not found!", label);
	return(0); // not found
}

int Bicounter::putData(count_t dataValA, count_t dataValB) {
	if(_nDataA >_maxNData) return FAIL;
	if(_nDataB >_maxNData) return FAIL;
	dataA[_nDataA] = dataValA;
	dataB[_nDataB] = dataValB;
	_nDataA++;
	_nDataB++;
	_expanded = false;
	return SUCCESS;
}

int Bicounter::setOutNumber(int number, char counter) {
	if(number > _maxNData) return FAIL;
	if(counter == 'A') _nDataA = number;
	else _nDataB = number;
	return SUCCESS;
}

int Bicounter::searchTestErrors(count_t test_number, char counter) {
	int test_errors = 0;
	int* nData;
	count_t* data;
	if(counter == 'A') {
		nData = &_nDataA;
		data = dataA;
	} else {
		nData = &_nDataB;
		data = dataB;
	}
	for(int i=0; i<*nData; i++) {
		test_number--;
		count_t derr = abs(data[i]-test_number);
		if(derr != 0) {
			test_errors += derr; 
			test_number = data[i];
		}
	}
	if((*nData) > 0) test_errors += data[(*nData)-1]; //yesli ne vse otshioty prishli
	return test_errors;
}

int Bicounter::searchTestErrors(count_t test_number) {
	return searchTestErrors(test_number,'A')+searchTestErrors(test_number,'B');
}

int Bicounter::setLevel(double level, char counter, int dialog) {
	jdebug("bicounter::setLevel");

	int levelValue =
		int(rint(255+_const2-rint(level*(255.0+_const2-_const1))));

	if(levelValue < MIN_LEVEL_VALUE) levelValue = MIN_LEVEL_VALUE;
	if(levelValue > MAX_LEVEL_VALUE) levelValue = MAX_LEVEL_VALUE;

	if(counter == 'A') {
		return setValue(uchar(levelValue),_levelValueA,_setLevelA,_getLevelA,
							"Error in setLevelA!", dialog);
	} else {
		return setValue(uchar(levelValue),_levelValueB,_setLevelB,_getLevelB,
							"Error in setLevelB!", dialog);
	}
}

double Bicounter::getLevel(char counter) {
	int levelValue;
	if(counter == 'A') levelValue = _levelValueA;
	else levelValue = _levelValueB;
	return double(255.0+_const2-levelValue)
					/ double(255.0+_const2-_const1);
}
