/*--------------------------------------------------------------------------*/
/** @file bicounter.h
@brief  Implementation of interaction (command calls and replies from modules, 
their current data and status keeping, etc.) with photometric modules.

                  MASS project: TURBINA module file
                       header file for bicounter.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.1
*/
#ifndef BICOUNTER_H
#define BICOUNTER_H

#include "procounter.h"
#include "iocount.h"

// Status of bicounter:

#define EXTERNAL	0x08	3// Integration using external clock
#define EXP_PAUSE	0x10	4// Microexposition end. Pause before the next
#define DATA_READY	0x40	6// New count in SRAM ready for transmit

class Bicounter : public ProCounter {
private:
	const char*	_channelA;
	const char*	_channelB;
	uchar _setLevelA;
	uchar _setLevelB;
	uchar _getLevelA;
	uchar _getLevelB;

	count_t		*dataA;
	count_t		*dataB;
	int			_nDataA;
	int			_nDataB;

	int			_levelValueA;
	int			_levelValueB;

	double		_workLevelA;
	double		_workLevelB;
	double		_nonLinearityA;
	double		_nonLinearityB;
	double		_nonPoissonA;
	double		_nonPoissonB;

	bool		_isCorrected;
	bool		_expanded;

public:
	Bicounter(void);
	virtual ~Bicounter();

	virtual void resetVariables(void);	// !!! virtual, t.k. vyzyvaetsia iz roditelia!!!
	void create(int sizeOfData);
	virtual int loadOwnConst(void);
	virtual int loadConst(void);

	void resizeData(int newSize);
	void resetData(void) {_nDataA=0;_nDataB=0;_nBlockLabel=0;_expanded=false;}
	/**
	@return	The fact that data were expanded due to the loss */
	bool expandData(void);
	int repairData(char counter);
	int insertBlock(int label, count_t value);
	int removeBlock(int label);
	int putData(count_t dataValA, count_t dataValB);
	count_t* getData(char counter) {if(counter == 'A') return(dataA);
									else return(dataB);}
	int getOutNumber(char counter) {if(counter == 'A') return(_nDataA);
									else return(_nDataB);}
	int setOutNumber(int number, char counter);
	int searchTestErrors(count_t test_number, char counter);
	int searchTestErrors(count_t test_number);

	int setLevel(double level, char counter, int dialog=1);
	double getLevel(char counter);	// stoit na samom dele
	double getWorkLevel(char counter) {	// dolzhen byt' (iz device.cfg)
							if(counter == 'A') return _workLevelA;
							else return _workLevelB;}
	double getNonPoisson(char counter) {if(counter == 'A') return _nonPoissonA;
										else return _nonPoissonB;}
	double getNonLinearity(char counter) {if(counter == 'A') return _nonLinearityA;
										else return _nonLinearityB;}
};

#endif
