//#define DEBUG_CNT		1
//#define DEBUG_STOP	1
//#define DEBUG_TIMING	1
//#define DEBUG_SPIKE	1
#define QT_THREAD_SUPPORT
#include <qthread.h>
#include <qdatetime.h>
#include <unistd.h>
#include "datathr.h"
#include "exchange.h"
#include "trbconst.h"

extern bool withDevice;

namespace datathr {
	unsigned char* data = NULL;
	int		n_read_data;
	int		block_number;
	int		_blockSize0;	// data + HEAD of block
//	int		_maxBlockSize;	// yesli prishel udlinennyi paket, on ! Ustarelo!!!
							// ne dolzhen byt' dlinnee

#ifdef __COUNTER_BUTTON__
	int			n_counters = 1;
	Counter*	counters[MAX_N_COUNTERS];
#else
	int			n_bicounters = 1;
	Bicounter*	bicounters[N_BICOUNTERS];
#endif
	bool	timeout_flag = false;
	bool	data_cycle_condition = true;
	FILE*	srv_file = NULL;
#ifdef DEBUG_SPIKE
	bool	spike = false;	// lovushka vybrosov
#endif

	QMutex	mutex;
	QMutex dataMutex;
}

void datathr::init() {
	n_read_data = 0;
	block_number = 512;
#ifdef __COUNTER_BUTTON__
	for(int i=0; i<MAX_N_COUNTERS; i++) counters[i]=NULL;
#else
	for(int i=0; i<N_BICOUNTERS; i++) bicounters[i]=NULL;
#endif
	srv_file = NULL;
}

void datathr::setSrvFile(FILE* f) {
	srv_file = f;
}

void datathr::setBlockNumber(int number) {
	block_number = number;
}

int datathr::getBlockNumber(void) {
	return block_number;
}

int datathr::getNblock(void) {
	if(!timeout_flag) {
#ifdef __COUNTER_BUTTON__
		return n_counters*block_number;
#else
		return n_bicounters*block_number*2;
#endif
	} else {
		return n_read_data;
	}
}

#ifdef __COUNTER_BUTTON__
void datathr::setNcounters(int number) {
	if(number <= MAX_N_COUNTERS) n_counters = number;
}

void datathr::setCounters(Counter** p_counters) {
	for(int i=0; i<MAX_N_COUNTERS; i++) {
		counters[i] = p_counters[i];
	}
}
#else
void datathr::setNbicounters(int number) {
	if(number <= N_BICOUNTERS) n_bicounters = number;
}

void datathr::setBicounters(Bicounter** p_bicounters) {
	for(int i=0; i<N_BICOUNTERS; i++) {
		bicounters[i] = p_bicounters[i];
	}
}
#endif


int datathr::setBlockSize(int size) {
	int blockSize = size;
	_blockSize0 = size+HEAD_SIZE;

	if(_blockSize0 >= CACHE_MAXSIZE) {
		_blockSize0 = CACHE_MAXSIZE-1;
		blockSize = _blockSize0-HEAD_SIZE;
	}
/*
	_maxBlockSize = _blockSize0 + HEAD_SIZE; //dlia udlinennykh paketov
	if(_maxBlockSize >= CACHE_MAXSIZE) {
		_maxBlockSize = CACHE_MAXSIZE-1;
		_blockSize0 = _maxBlockSize - HEAD_SIZE;
		blockSize = _blockSize0-HEAD_SIZE;
	}
*/
	return blockSize;
}

int datathr::getNreadData(void) {
	return n_read_data;
}

void datathr::startCounting(void) {
    n_read_data = 0;
}

void datathr::setTimeout(bool tmout) {
	timeout_flag = tmout;
}

bool datathr::isTimeout(void) {
	return timeout_flag;
}

void datathr::setCycleCondition(bool condition) {
	mutex.lock();
	data_cycle_condition = condition;
	mutex.unlock();
}

bool datathr::getCycleCondition(void) {
	mutex.lock();
	bool condition = data_cycle_condition;
	mutex.unlock();
	return condition;
}



void datathr::quitDataCycle(void) {
//	jdebug("quitDataCycle");
	setCycleCondition(false);
	if(dataMutex.locked()) dataMutex.unlock();
//	jdebug("quitDataCycle: dataMutex unlocked");
}

void datathr::timeoutAction(void) {
	timeout_flag = true;
	if(srv_file) fprintf(srv_file,"timeout_flag=%d\n",timeout_flag);
	if(dataMutex.locked()) dataMutex.unlock();
//	jdebug("dataMutex unlocked");
	if(srv_file) fprintf(srv_file,"dataMutex unlocked\n");

// debug!!!
//			printf("Stop!!!! timeout!!!!\n");
//			sendCommand("0x1 0x81");
//			exit(1);
}

void datathr::read_data(void) {		// dataThread
//	atexit(quitDataCycle);
	dataMutex.lock();
// DEBUG !!!
	static QTime t_prev;

//	jdebug("+++++++++++++++++++ read_data: Start...");
	n_read_data = 0;

	int count_byte = 0;
	while(getCycleCondition() && withDevice) {
#ifdef __COUNTER_BUTTON__
		while(n_read_data < n_counters*block_number) {
			if(!getCycleCondition() || !withDevice) break;
			count_byte = exch::readPacket(&data);
			if(count_byte <= 0) {
				usleep(10000);
				continue;
			}
// DEBUG!!!
#ifdef DEBUG_TIMING
			QTime dt = QTime::currentTime();
			
			int dmsec = -1*dt.msecsTo(t_prev);
			if((dmsec>100) && srv_file) fprintf(srv_file,"big interval %4d msec!\n",
						dmsec);
			t_prev = dt;
#endif

			putData();

#ifdef DEBUG_CNT
			if(srv_file) {
//				fprintf(srv_file,"%4d  ", dmsec);
				for(int t=0; t<=count_byte; t++) {
					fprintf(srv_file,"%02X ",data[t]);
				}
				int blnum = exch::twoUcharToUint(data[3],data[4]);
				fprintf(srv_file," %d %d\n",n_read_data,blnum);
			}
#endif
#ifdef DEBUG_STOP
			if(data[1] != 0) {
				printf("Stop!!!! signature = %d\n", data[1]);
				sendCommand(0x1,0x81);
				exit(1);
			}
#endif
			n_read_data++;
		}
#else
		while(n_read_data < n_bicounters*block_number*2) {
			if(!getCycleCondition() || !withDevice) break;
			count_byte = exch::readPacket(&data);	// ot 2-x chiotchikov!
			if(count_byte <= 0) {
				usleep(10000);
				continue;
			}

			putData();
#ifdef DEBUG_CNT
			if(srv_file) {
				for(int t=0; t<=count_byte; t++) {
					fprintf(srv_file,"%02X ",data[t]);
				}
				int blnum = exch::twoUcharToUint(data[3],data[4]);
				fprintf(srv_file," %d %d\n",n_read_data,blnum);
			}
#endif
#ifdef DEBUG_STOP
			if(data[1] != 0) {
				printf("Stop!!!! signature = %d\n", data[1]);
				sendCommand(0x1,0x81);
				exit(1);
			}
#endif
			n_read_data++;
		}
#endif
		n_read_data = 0;
		if(dataMutex.locked()) dataMutex.unlock();
	}
	if(dataMutex.locked()) dataMutex.unlock();
//	jdebug("Last: dataMutex unlocked");
//	return(1);
}

#ifdef __COUNTER_BUTTON__
void datathr::putData(void) {
	Counter *currentCounter = NULL;
	for(int i=0; i<MAX_N_COUNTERS; i++) {
		if(counters[i])
			if(data[2] == (uchar)counters[i]->getModule())
				currentCounter = counters[i];
	}

	if(!currentCounter) {
		if(srv_file) fprintf(srv_file, "Unknown counter with address %X\n",data[2]);

//			printf("Stop!!!!unknown counter!!!\n");
//			sendCommand("0x1 0x81");
//			exit(1);

		return;
	}

	uchar size = data[0];

	bool ok = true;
	if(size != (_blockSize0-1)) {
		ok = false;
	}

	int blnumber = exch::twoUcharToUint(data[3],data[4]);

	int dnumb = abs(blnumber - currentCounter->blockCounter.get());
	if(dnumb != 0) {
		if(srv_file) fprintf(srv_file,"%s block %02X-1 %02X skipped\n",
				currentCounter->getName(),data[3],data[4]);

//			printf("Stop!!!! block skipped!!!\n");
//			sendCommand("0x1 0x81");
//			exit(1);

		currentCounter->blockCounter.set(blnumber);
	}

// LOVUSHKA vybrosov:
#ifdef DEBUG_SPIKE
	const int test = 0;
	const int maxVal = 1000;
	if(currentCounter->getModule() == counters[test]->getModule()) {
		spike = false;
	}
#endif
	if(ok) {
		currentCounter->addBlockLabel(blnumber);
		for(uchar t=HEAD_SIZE; t<=size; t=t+2) {
//			count_t val = data[t]+(data[t+1] << 8);
			count_t val = exch::twoUcharToUint(data[t],data[t+1]);
			currentCounter->putData(val);

//	Yest' vybros???
#ifdef DEBUG_SPIKE
			if(currentCounter->getModule() == counters[test]->getModule()) {
				if(val >= maxVal) spike = true;
			}
#endif
		}
#ifdef DEBUG_SPIKE
		if(spike && srv_file) {
			for(int t=0; t<=size; t++) {
				fprintf(srv_file,"%02X ",data[t]);
			}
			fprintf(srv_file,"\n");
		}
#endif
		currentCounter->blockCounter.incr();
	} else {
/*
		jdebug("!!!!!!!!!!!!!!%s: Skip block !!!!!!!",
				currentCounter->getName());
*/
		if(srv_file) fprintf(srv_file,"%s Skip block %02X %02X\n",
				currentCounter->getName(),data[3],data[4]);
	}
}

#else

void datathr::putData(void) {
	Bicounter *currentBicounter = NULL;
	for(int i=0; i<N_BICOUNTERS; i++) {
		if(bicounters[i])
			if(data[2] == (uchar)bicounters[i]->getModule())
				currentBicounter = bicounters[i];
	}

	if(!currentBicounter) {
		if(srv_file) fprintf(srv_file, "Unknown bicounter with address %X\n",data[2]);
		return;
	}

	uchar size = data[0];

	bool ok = true;
	if(size != (_blockSize0-1)) {
		ok = false;
	}

	int blnumber = exch::twoUcharToUint(data[3],data[4]);

	int dnumb = abs(blnumber - currentBicounter->blockCounter.get());
	if(dnumb != 0) {
		if(srv_file) fprintf(srv_file,"%s block %02X-1 %02X skipped\n",
				currentBicounter->getName(),data[3],data[4]);

		currentBicounter->blockCounter.set(blnumber);
	}

#ifdef DEBUG_SPIKE
//	const int test = 1;
	const int maxVal0A = 6000;
	const int maxVal0B = 6000;
	const int maxVal1A = 6000;
	const int maxVal1B = 6000;
//	if(currentBicounter->getModule() == bicounters[test]->getModule()) {
		spike = false;
//	}
#endif
	if(ok) {
		currentBicounter->addBlockLabel(blnumber);
#ifdef DEBUG_SPIKE
		count_t spikeA = 0;
		count_t spikeB = 0;
#endif
		for(uchar t=HEAD_SIZE; t<=size; t=t+4) {
			count_t valA = exch::twoUcharToUint(data[t],data[t+1]);
			count_t valB = exch::twoUcharToUint(data[t+2],data[t+3]);
//				if(srv_file) fprintf(srv_file,"|%d %d|",valA,valB);
			currentBicounter->putData(valA,valB);
#ifdef DEBUG_SPIKE
			if(currentBicounter->getModule() == bicounters[0]->getModule()) {
				if(valA >= maxVal0A) {spike = true; spikeA = valA;}
				if(valB >= maxVal0B) {spike = true; spikeB = valB;}
			}
			if(currentBicounter->getModule() == bicounters[1]->getModule()) {
				if(valA >= maxVal1A) {spike = true; spikeA = valA;}
				if(valB >= maxVal1B) {spike = true; spikeB = valB;}
			}
#endif
		}
//		if(srv_file) fprintf(srv_file,"\n");
#ifdef DEBUG_SPIKE
		if(spike && srv_file) {
			fprintf(srv_file, "Spike = |%d %d|:\n",spikeA, spikeB);
			for(int t=0; t<=size; t++) {
				fprintf(srv_file,"%02X ",data[t]);
			}
			fprintf(srv_file,"\n");
		}
#endif
		currentBicounter->blockCounter.incr();
	} else {
		if(srv_file) fprintf(srv_file,"%s Skip block %02X %02X\n",
				currentBicounter->getName(),data[3],data[4]);
	}
}
#endif

void datathr::waitData(void) {
	if(srv_file) fprintf(srv_file,"waitData...\n");
	if(!dataMutex.locked()) {
		if(srv_file) fprintf(srv_file,"waitData: dataMutex.lock()-2\n");
		dataMutex.lock(); // if unlock
		if(srv_file) fprintf(srv_file,"waitData: dataMutex locked-2\n");
	}
	dataMutex.lock();
	if(srv_file) fprintf(srv_file,"waitData: dataMutex locked\n");
	if(isTimeout()) {
#ifdef __COUNTER_BUTTON__
		for(int i=0; i<MAX_N_COUNTERS; i++) {
			counters[i]->stop();
		}
#else
		for(int i=0; i<N_BICOUNTERS; i++) {
			bicounters[i]->stop();
		}
#endif
	}
}
