/*--------------------------------------------------------------------------*/
/** @file graph.h
@brief  Supports the output of the measurement results in the graphic window.

                  MASS project: TURBINA module file
                       header file for graph.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef GRAPH_H
#define GRAPH_H

#include <qpainter.h>
#include <qpixmap.h>
#include <qwidget.h>

#include "outcontrol.h"

struct Scale {
	Scale::Scale(double xMn, double xMx,
				double yMn, double yMx) {
				xmin = xMn;
				xmax = xMx;
				ymin = yMn;
				ymax = yMx;
			}
	double xmin;
	double xmax;
	double ymin;
	double ymax;
	double sx;
	double sy;
};

class GraphWindow : public QWidget {
	Q_OBJECT
private:
	QPixmap _buffer;

	bool	empty;

	int	_winX;
	int	_winY;
	int	_winWidth;
	int	_winHeight;
	int	_border_y;
	int	_view_marg;
	QFont	_font;
	int	_fontSize;
	int	_pointSize;
	QColor	_background;
	QColor	_axesColor;

	int	timer;
	bool	isTimerEventLocked;

	bool	_showLut;
	bool	_reDrawLut;
	double	_maxLutVal;
	double	_minLutVal;
public:
	GraphWindow(QWidget *parent=0, const char *name=0);
	~GraphWindow(void);

private:
	void loadConst(bool first=true);

	void drawX(QPainter*,
			Scale*,
			int,
			bool top=false,
			const char* name="");
	void drawX(QPainter*);
	void drawY(QPainter*,
			Scale*,
			int,
			bool left,
			bool drawNull=false,
			const char* name="",
			bool logY=false);

	void drawViewPort(GraphSet* gs,
				QPainter* bp,
				int left,
				int top,
				int width,
				int height,
				bool reDraw,
				bool drawTop=false,
				bool drawDawn=false);
	void setLutColor(QPainter* p, double val, double maxVal);
	void drawCurve(GraphSet* gs,
			QPainter* bp,
			Scale* s,
			int wt,
			int wr,
			bool reDraw = true);
	void drawLut(QPainter* bp,
			int left,
			int top,
			int width,
			int height,
			double minVal,
			double maxVal);
	void drawAll(bool reDraw);

private slots:
	void change(void);

protected:
	void timerEvent(QTimerEvent*);
	virtual void paintEvent(QPaintEvent*);
	virtual void resizeEvent(QResizeEvent*);
//	virtual void moveEvent(QMoveEvent*);
//	virtual void mousePressEvent(QMouseEvent*);
//	virtual void mouseMoveEvent(QMouseEvent*);
};

#endif
