#include <iostream>	// for testing only
#include <limits.h>	// for testing only

#include "iocount.h"
#include "nrutil.h"


#ifdef IOCOUNTTEST

int main() 
{
	cout << "Sizeof(count_t): " << sizeof(count_t) << " bytes\n" ;

	cout << "Max count_t = " << max_count_t << '\n';
	FILE * f = fopen("tmp.cnt","a+b") ;
	int ndata = 1002 ;
	int nbuf = 4 ;
	count_t ** data = new (count_t*)[nbuf] ;
	for (int b=0; b<nbuf; b++) data[b] = new count_t[ndata] ;
//	count_t data[nbuf][ndata] ;
	
	char select = 1;
	while (select) {
	cout << "Write/read series or exit? (w/r/<any>): " ;
	cin >> select ;
	switch (select) {
		case 'w': 
		{
		cout << "Number of series: " ;
		int nseries = 0;
		cin >> nseries ;
		for (int ser=0 ; ser<nseries; ser++) {
			for (int b=0; b<nbuf; b++) 
				for (int i=0; i<ndata; i++) data[b][i] = (i<<b)+ser*100 ;

			long addr = ioc::write(f, data, ndata, nbuf) ;
			if (nr::erget()) {
				cerr << nr::ermessage() << '\n' ;
				exit (1);
			}
			cout << "Series #" << ser;
			cout.setf(ios::hex,ios::basefield);
			cout << " is written at " << addr << '\n' ;
			cout.setf(ios::dec,ios::basefield);
		}
		break ;
		}
		
		case 'r' :
		{
		cout << "Enter the exact hexadecimal address returned by write(): " ;
		long address ;
		cin.setf(ios::hex,ios::basefield);
		cin >> address ;
		cin.setf(ios::dec,ios::basefield);

		ioc::read(f, (count_t**)data, &ndata, &nbuf, address) ;
		if (nr::erget()) {
			cerr << nr::ermessage() << '\n' ;
			cerr << "Nbuf, Ndata read are: " << nbuf << ',' << ndata << '\n';
			exit (1);
		} else {
			for (int b=0 ; b<nbuf; b++) {
				count_t min = SHRT_MAX, max = 0 ;
				for (int i=0; i<ndata; i++) {
					if (data[b][i]<min) min=data[b][i] ;
					if (data[b][i]>max) max=data[b][i] ;
				}
				cout << "Series["<<b<<"] has data within [" << min << ',' 
				 	<< max << "]\n" ;
			}
		}
		break ;
		}

		default:
			select = 0 ;
	}	// end of switch
	}
	fclose(f) ;
}
#endif

namespace ioc {
const long badaddr = -1 ;	// to be returned from write() on I/O failure
const int descrlen = 2 ;	// length of the buffer descriptor {Nbuf,Ndata}
}

long ioc::write(FILE * fcnt, count_t ** buffer, int buflen, int nbuf) {

	/* Goto the end of file: */
	int result = fseek(fcnt, 0L, SEEK_END) ;
	
	/* Get the current position of the count stream: */
	long posstart = ftell(fcnt) ;
	if (posstart<0 || result) {
		nr::nrerror("IOCOUNT: cannot set position in count file", nr::ERFIO) ;
		return badaddr ;
	} 

	const size_t size = sizeof(count_t) ;
	count_t descr[descrlen] ;
	descr[0] = nbuf ;
	descr[1] = buflen ;
	for (int b=0; b<nbuf; b++) {
		/* write the two-numbers descriptor: */
		int written = fwrite(descr, size, descrlen, fcnt) ;
		if (written==descrlen) 
			written = fwrite(buffer[b], size, buflen, fcnt) ;
		if (written < buflen) {
			nr::nrerror("IOCOUNT: cannot write() count file", nr::ERFIO) ;
			return badaddr ;
		}
		(descr[0])-- ;
	}
	return posstart ;
}

void ioc::read(FILE * fcnt, count_t ** buffer, int * buflen, int * nbuf, 
	long addr) {

	/* Goto the needed address: */
	int result = fseek(fcnt, addr, SEEK_SET) ;

	const size_t size = sizeof(count_t) ;

	if (addr % size) {
		nr::nrerror("IOCOUNT: read() with address asynchronous with count-type",
			nr::ERPAR) ;
		return ;
	}
	
	/* Get the current position of the count stream: */
	long posstart = ftell(fcnt) ;
	if (posstart!=addr || result) {
		nr::nrerror("IOCOUNT: read() cannot set position in count file",nr::ERFIO) ;
		return ;
	}

	count_t descr[descrlen] ;

	for (int b=0; b<(*nbuf); b++) {
		/* read the descriptor: */
		int rd = fread(descr, size, descrlen, fcnt) ;
		if (rd < descrlen) {
			nr::nrerror("IOCOUNT: cannot read() from count file",nr::ERFIO) ;
			return ;
		} 
		/* Check and return the descriptor content: */
		if (descr[0]!=(*nbuf)-b) {
			/* either synchro error or something else: */
			nr::nrerror("IOCOUNT: Invalid buffer sequential number",
				nr::EROFL) ;
		}
		if (b==0) {
			if ((descr[1]>(*buflen)) || (descr[1] <= 0)) {
				nr::nrerror("IOCOUNT: Invalid value of buffer length descriptor",
				nr::EROFL) ;
			}
			*nbuf = descr[0] ;	// save descriptor values in place-holders
			*buflen = descr[1] ;
		} else {
			if (*buflen!=descr[1])
				nr::nrerror("IOCOUNT: unequal values of buffer length descriptor",
				nr::EROFL) ;
		}
		if (nr::erget()) return ;	// return if one of errors occured
		rd = fread(buffer[b], size, *buflen, fcnt) ;
		if (rd < (*buflen)) {
			nr::nrerror("IOCOUNT: cannot read() from count file",nr::ERFIO) ;
			return ;
		} 
	}
}
