/*--------------------------------------------------------------------------*/
/**
@mainpage	MASS Software. Part I: Instrument Control

\section introd Introduction

The TURBINA program represents the GUI-interface for managing the
measurements of stellar scintillation indices and atmospheric integral
parameters and turbulence profiles. It is written in C++ language and is
based on the Qt libraries which provide the means for creation of GUI
components. In this part of the document, the principles of the program
organization, its modules and their mutual relations and dependencies are
described. The "scientific" modules which deal with scintillation and
atmospheric calculations are described in a second part of MASS Software
Reference "Data Processing".

TURBINA is a complex program executed in X-Windows environment which provides
the interface between the User and the device or, more precisely, its driver
inserted in the Linux kernel on the stage of the PC startup. Its actions and
events are naturally asynchronous; the calculations are done in real-time, in
parallel with interaction with user and device and I/O operations with disk.
Apart from this, the CPU time cannot be fully consumed by TURBINA and must be
left for other running system and user's applications. So, the program is
organized in a complex way which demands an introduction of some basic notions.
They are not specific to TURBINA only and used in most of similar applications,
but it is useful to describe them in brief here.

\section notions Basic notions

In this section, we give a short explanation for "thread" and "mutex" notions
on which the TURBINA operation is based. 

- \em Thread is a separate sequence of CPU instructions which serve for one
particular task of application and which can work in parallel with other
threads. Technically, the thread owns its stack and CPU registers sets
which are loaded in CPU when it is under execution. Meanwhile, all the threads
started from one program (i.e. \em process) operate on the same memory segment 
which demands some means for synchronization of the data access between them 
(see below).

From the point of view of TURBINA software, the \em threads which are started
explicitly are realized as infinite cycles. These are the 
	\arg the "Scenario" thread which implements the execution of modes
or their sequences (\em scenario);
	\arg the "Data" thread which waits for data blocks from the driver FIFO and
sorts the received data into four buffers 	associated with the MASS channels
(A, B, C and D).

Two other threads which are started implicitly on the program startup are the
"Main" GUI-thread which serves for the interaction with a user, and another
("internal") thread which serves the GUI system.

Since the threads within one program - i.e. TURBINA - share the same memory
segment and need to access the same data structures for exchanging the
information between them, some \em synchronization of this access from parallel
running threads is needed. It is made by means of \em Mutex.

- \em Mutex is a structure in memory (Qt represents the class "QMutex") which
serves for threads manipulation:
	\arg for their synchronization and for preventing the conflicts while
	occasionally simultaneously accessing the same  data structures under
	modification.
	\arg for stopping/resuming the working of threads according to different
circumstances occurring in the program 	execution.

To implement these functions, the mutex has the ("atomic", i.e.
non-interruptible) functions \b lock() - to stop execution of (some) thread,
and \b unlock() - to resume execution of a thread. In TURBINA, the thread
stops itself with the help of the mutex lock() and is resumed from \em
another thread by unlock()-ing of the same mutex. The following rules describe the
behavior of the program (one of its threads) when its control reaches the mutex
lock/unlock instruction:
	\arg lock() on the mutex (variable) which is unlocked results in locking this
mutex and the control goes to the next instruction after lock(), i.e. the
thread continues to work;
	\arg lock() on the mutex which is \em already locked() stops the current
thread execution until some other thread unlocks this mutex. Then the previous
rule takes place (locking and continuation).
	\arg unlock() on mutex in any state unlocks it and the control goes further.

In what follows, we will explain the place of these notions in TURBINA
organization.

\section threads Justification of the threads use and their interaction

The key notions presented above help to understand the overall scheme of
working of TURBINA (see Figure 1 below) in execution of its basic functions -
reaction to the user's actions and digesting the input data from the device.

After the program startup (optionally, Initial Scenario is performed), the
control is reserved completely for Main thread. It waits for actions
from user (clicking on buttons or menu items etc.) in reply to which it

- sends some commands directly to the device (i.e. driver). As example - "Open
Door", "Turn HV On", etc., and waits until these commands are completed.

- starts the different modes or their sequences (scenarii) setting the mode
switch to the needed mode or to the first mode of scenario. This switch
(squares with slash in the figure) is passed by the "Scenario" thread control
when the "Main" thread unlocks the respective mutex in it. Then the "Scenario"
thread selects the mode in a switch and calls one of the respective mode 
procedures (implemented in \b device module). This algorithm (see next Figure 2)
initializes itself, sends some commands to the device to start the accumulation
of data and waits until these data are ready in the mutex lock point (black box
in each mode algorithm rectangle).

\image latex turb_thread.eps	"Interaction of the three TURBINA threads shown as three large marked rectangles. Black boxes represent mutexes which are unlocked by incoming arrows from other threads and locked by internal cycles of threads." width=16cm
\image html  turb_thread.gif	"Fig.1. Interaction of the three TURBINA threads shown as three large marked rectangles. Black boxes represent mutexes which are unlocked by incoming arrows from other threads and locked by internal cycles of threads."

\image latex turb_mode.eps	"Block-scheme of a mode algorithm. The cycle is over the  base-time and the loops are repeated approximately (accumulation time)/(base time) times. So, the scheme describes one full accumulation time actions." height=10cm
\image html  turb_mode.gif	"Fig.2. Block-scheme of a mode algorithm. The cycle is over the  base-time and the loops are repeated approximately (accumulation time)/(base time) times. So, the scheme describes one full accumulation time actions." 

When the acquired data arrive in the driver FIFO from the device modules (thick
arrow from the device box to "Data" thread on the Fig.1), they are
sorted by MASS channel buffers in "Data" thread and, when ready, this thread
unlocks the mutex which has stopped the mode in "Scenario" thread. The data are
reduced (see Fig.2), displayed, stored etc. and thus one \b
accumulation time cycle is over. 

Then the control goes to the point where the "next mode" is selected and set in
a mode switch \em if the scenario was started. If it was a single mode call from
the "Main" thread, the control stops at the mutex lock point until the next
call from the "Main" thread.

\section modules The TURBINA modules and their dependencies

From the point of view of programming, the program consists of a number of
modules (compiled independently and linked later in one executable file). The
scheme of their mutual relations is presented in Fig.3 where arrows show the
"inclusion way": an arrow going from module 1 to module 2 means that module 1
is used by (or \em #include-d in) the module 2.

\image latex turb_dep.eps "The tree of module dependencies in TURBINA. Light grey/yellow boxes - scientific modules, grey/cyan boxes - GUI modules." width=16cm
\image html  turb_dep.gif "Fig.3. The tree of module dependencies in TURBINA. Yellow boxes - scientific modules, cyan boxes - GUI modules."

\b References

1. UNIX System Programming using C++, T. Chan, Prentice Hall PTR, NJ, 1997

2. Linux device drivers. A. Rubini, O'Reilly, 1998

3. The C++ Programming Language. B. Stroustrup. Third Edition.
Addison-Wesley, 2001

4. Programming with Qt. M.K.Dalheimer. O'Reilly, 1999 
[obsolete edition used for principles study only]

5. HTML reference manual pages for Qt programming at {\tt 
http://doc.trolltech.com}

*/
/** @file main.cpp
@brief  Initiates the program execution.

                  MASS project: TURBINA main file

 This is a top-level module of TURBINA program, a starting point of a
program where the following functions are executed.
- Reservation of static variables for trbexport.h,
- start of CFG files reading into the trees in memory,
- start the "Scenario" and "Data" threads,
- initialize the device (if \c turbina.cfg contains "Yes" in \c
Operations/Common/WithDevice),
- create the main and graphic windows, load the module constants,
- If Initscenario is activated in CFG, start this scenario in the "Scenario"
thread (all the other similar starts are made via GUI).

Since many of the class constructors (from other modules) are (implicitly)
started here, on the stage of the program startup all the checks of consistency
are done (e.g. existence and validity of weight function file for
all stars in the list, etc.).

 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/

#include <qapplication.h>
#include <qfont.h> 
#include <qdatetime.h>
#include "trbthread.h"
#include "scenariothr.h"
#include "exchange.h"
#include "names.h"
#include "device.h"
#include "cfg.h"
#include "trbtime.h"
#include "graph.h"
#include "trbexport.h"
#include "access.h"
#include "outcontrol.h"
#include "star.h"
#include "mywidget.h"

bool		ExportText::isChanged = false;
QString		ExportText::text;

bool		ExportTable::clean=false;
bool		ExportTable::adjust=false;
bool		ExportTable::isChanged=false;
int			ExportTable::startRow = 0;
int			ExportTable::nRows = MAX_ROW;
int			ExportTable::startCol = 0;
int			ExportTable::nCols = MAX_COL;
QString		ExportTable::table[MAX_ROW][MAX_COL];

bool		InfoExport::isMessage = false;
QString		InfoExport::message;
int			InfoExport::ms;	// skol'ko emu svetit'sia, ms
bool		InfoExport::isProgressChanged = false;
Progress	InfoExport::progress;
bool		InfoExport::isModeChanged = false;
Progress	InfoExport::mode;
bool		InfoExport::withSV = false;
bool		InfoExport::isScenarioChanged = false;
bool		InfoExport::isScenarioStateChanged = false;
bool		InfoExport::isScenario = false;
Progress	InfoExport::scenario;
bool		InfoExport::isStarPositionChanged = false;
int			InfoExport::starPosition;

QMutex infoExportMutex;

DevManager	devmanager;
Star		star;
OutControl	outcontrol;
ExportTable	table;
FILE		*out_file=NULL;
FILE		*stm_file=NULL;
bool		isMassRoot;
bool		withDevice=false;

CFGBranch branch_empty("Empty",(CFGTree*) NULL);
CFGLeaf	leaf_empty("Empty",&branch_empty);

CFGTree		_tree;		//  
CFGTree*	tree = &_tree;
CFGTree		_devicetree;
CFGTree		*devicetree = &_devicetree;

int main( int argc, char ** argv ) {
	printf("                       TURBINA Ver.%s\n",VERSION);
	isMassRoot = getMassAccess();

//	----- Tree:

	QApplication qapp( argc, argv );

	if( _devicetree.readCfg(deviceCfgFile, NULL) ) exit(1);
	if( _tree.readCfg(cfgFile, cfgStatusFile) ) exit(1);

	leafDataOnOffToInt(withDevice, true, cfgFile,"Operations",
					"Common","WithDevice" );

	exch::init(withDevice);
	trbthread::create();
	trbthread::start();

	trb_tm::loadConst();

	devmanager.create();
	devmanager.prepareFiles();

	star.loadConst();
	star.init(1);

	outcontrol.loadConst();
	devmanager.loadConst();

	MyWidget w( 0,"TextWindow" );
	qapp.setMainWidget( &w );
	GraphWindow gw( 0,"GraphWindow" );
	gw.show();
	w.show();

//VK	if(withDevice) devmanager.init();
//VK	if(withDevice) scenariothr::playInitialScenario();

	return qapp.exec();
}
