#ifndef MODULE_H
#define MODULE_H

#define MIN_BYTE_VALUE	0
#define MAX_BYTE_VALUE	255
#define MIN_SIGN_BYTE_VALUE		(-127)
#define MAX_SIGN_BYTE_VALUE		127

typedef	unsigned char uchar;
typedef unsigned int uint;     

class HardModule {

protected:

	const char* _name; 
	const char* _identName;
	const char* _identSection;

	uchar _module;
	uchar _getCRC;
	uchar _getStatus;
	uchar _getIdent;
	int	  _ident1,_ident2,_ident3,_ident4;
	uchar _getConst;
	int	  _const1,_const2,_const3,_const4;
	uchar _reset;
	uchar _testMotion;
	uchar _stopMotion;

	bool  _ready;

private:
	template <class X> int setValue_(X value, int& currentValue,
					uchar setCommand, uchar getCommand,
					const char* failMessage,
					int dialog=1, int waitTime=0);

public:
	HardModule(void);
	virtual ~HardModule();

	virtual void resetVariables(void) = 0;
	int loadOwnConst(void);

	bool isReady(void) { return(_ready); }

	int reset(int dialog=1);
	int getConst(int dialog=1);
	int testIdent(int dialog=1);
	int getValue(uchar& value, uchar getCommad, int dialog=1, bool saveToLog=true);
	int getValue(int& value, uchar getCommad, int dialog=1, bool saveToLog=true);
	int sendCommand(uchar command, int dialog=1);
	int sendValue(uchar command, uchar value, int dialog=1);
	int sendValue(uchar command, int value, int dialog=1);
	int sendValue(uchar command, uchar value1, uchar value2, int dialog=1);
	int turn(int state,int& currentState,	// state = ON or OFF
					uchar setOn, uchar setOff,
					int dialog=1,int waitTime=0);
	int setValue(uchar value, int& currentValue,
					uchar setCommand, uchar getCommand,
					const char* failMessage,
					int dialog=1, int waitTime=0);
	int setValue(int value, int& currentValue,
					uchar setCommand, uchar getCommand,
					const char* failMessage,
					int dialog=1, int waitTime=0);
	int testCRC(int dialog=1);
	int getStatus(uchar& status, int dialog=1, bool saveToLog=true);
	int isMotion(int dialog=1);
	int stopMotion(int dialog=1);
};

#endif
