#define QT_THREAD_SUPPORT
#include <math.h>
#include <qthread.h>
#include <qdir.h>
#include <qtextstream.h>
#include "outcontrol.h"
#include "names.h"
#include "tools.h"
#include "messages.h"
#include "cfg.h"
#include "tree.h"
#include "nrutil.h"
#include "trberror.h"
#include "trbexport.h"
#include "trbconst.h"
#ifdef __MOTOR__
	#include "motor.h"	// for MAX_SYNCH_SHIFT
#elif defined __SCANMOTOR__
	#include "scanmotor.h"
#endif

extern CFGTree* tree;
extern OutControl outcontrol;

#if defined __MOTOR__ || defined __SCANMOTOR__
	#define MAX_SCAN	(MAX_SYNCH_SHIFT+1)
	#define MAX_CHANNEL	N_CHAN
	static double scanX[MAX_SCAN];
	static double scanY[MAX_CHANNEL][MAX_SCAN];
	static int scanLength = 0;
#endif

static std::string pnameX = "UT";

QMutex graphExportMutex;

void GraphSet::load(QString& pname_,
			const char* color_,
			int size_,
//			sum::reftype rtype,
			double refer,
			double* x,
			double* y,
			int nP,
			double scale) {
	_pname = pname_;
	_color = color_;
	_size = size_;

//	double refVal = sum::NODATA;

/*
	if(isProfile()) {
		refVal = scale * sum::getref("FM0", rtype); 
	} else if(!isScan())
		refVal = sum::getref(_pname.data(), rtype);
	if(nr::erget() == nr::ERNOD) {
		QString mess = nr::ermessage().c_str();
		showStrongMessage(mess);
		nr::erreset();
	} else if(refVal != sum::NODATA) _refer = refVal;
*/
	_refer = refer;
	if(isScan()) _refer = sum::NODATA;
//	printf("%s: refer=%g\n",_pname.data(),_refer);

	setData(x,y,nP);
}

void GraphSet::setData(double* x, double* y, int nP) { 
	_xData = x;
	_yData = y;
	_nPoint = nP;
}

void GraphSet::setNPoint(int nP) { 
	graphExportMutex.lock();	
	_nPoint = nP;
	graphExportMutex.unlock();	
}

void GraphSet::setNPointGraph(int nP) { 
	graphExportMutex.lock();	
	_nPointGraph = nP;
	graphExportMutex.unlock();	
}

#if defined __MOTOR__ || defined __SCANMOTOR__
void GraphSet::copyScan(double* y, int startX, int nP) { 
	if(isScan() && (nP >= MAX_SCAN)) nP = MAX_SCAN-1;

	graphExportMutex.lock();	
	for(int i=0; i<nP; i++) {
		_xData[i] = i + startX;
		_yData[i] = y[i];
	}
	_nPoint = nP;
	_nPointGraph = 0;
	scanLength = nP;
	graphExportMutex.unlock();	

}
#endif

const double T_FACT = 1.5;

bool GraphSet::searchMinMax(void) {
	double xMinOld = _xMin;
	double xMaxOld = _xMax;
	double yMinOld = _yMin;
	double yMaxOld = _yMax;
	if(_nPoint == 0) {
		_xMin = 0;
		_xMax = 0;
		_yMin = 0;
		_yMax = 0;
		return true;	
	}
// Y:
	_yMin = 0;
	_yMax = 0;
	if(isProfile()) {
		if(!outcontrol.isPercent()) {
			_yMin = outcontrol.zSmooth()[0];
			_yMax = outcontrol.zSmooth()[outcontrol.nZsmooth()-1];
		} else {
			for(int i=0; i<_nPoint; i++) {
				int nCN2 = (int)sum::getdata("NCN2")[i];
				if(nCN2 != sum::NODATA) {
					double max_ = 
						sum::getdata("ZCN2",i)[nCN2-1];
					if(max_ > _yMax) _yMax = max_;
				}
			}
		}
	} else if(isScan()) {
		for(int i=0; i<_nPoint; i++) {
			if((_yData[i] != sum::NODATA)
			&& (_yData[i] < _yMin)) _yMin = _yData[i];
			if((_yData[i] != sum::NODATA)
				&& (_yData[i] > _yMax)) _yMax = _yData[i];
		}
	} else {
		if(_refer != sum::NODATA) _yMax = _refer * 2.0;
		else _yMax = 1.0;
// Ne khotiat tak v "antisanitarnykh usloviyakh" - nu i ne nado,
// ne ochen'-to i khotelos'...
/*
		for(int i=0; i<_nPoint; i++) {
			if((_yData[i] != sum::NODATA)
					&& (_yData[i] > _yMax)) {
				bool change = true;
				for(int j=i; j>i-3; j--) {
					if(j<0) break;
					if(_yData[j] <= _yMax) {
				// khot' odin iz predyduchikh triokh < max
						change = false;
						break;
					}
				}
				if(change) _yMax = _yData[i];
			}
		}
*/
	}
// X:
	if(isScan()) {
		_xMin = _xData[0];
		_xMax = _xData[_nPoint-1];
		for(int i=0; i<_nPoint; i++) {
			if((_yData[i] != sum::NODATA)
			&& (_yData[i] < _yMin)) _yMin = _yData[i];
			if((_yData[i] != sum::NODATA)
				&& (_yData[i] > _yMax)) _yMax = _yData[i];
		}
	} else {
		_xMin = _xData[0];
		_xMax = _xMin + 2.0;	// init: 2h
		while(_xData[_nPoint-1] > _xMax) {
			_xMax = (_xMax-_xMin) * T_FACT + _xMin;
		}
	}

	bool changed = ((_xMin != xMinOld) ||
				(_xMax != xMaxOld) ||
				(_yMin != yMinOld) ||
				(_yMax != yMaxOld));
	return changed;
}

bool GraphSet::isScan(void) {
	if(_pname.contains("S_")) return true;
	else return false;
}

bool GraphSet::isProfile(void) {
	if(_pname == "CN2") return true;
	else return false;
}

void GraphData::add(QString& pname,
			const char* color,
			int size,
//			sum::reftype rtype,
			double refer,
			double* xData,
			double* yData,
			int nPoint,
			double scale) {

	graphSet.resize(_nData+1);
	graphSet[_nData] = new GraphSet();
//	graphSet[_nData]->load(pname,color,size,rtype,
	graphSet[_nData]->load(pname,color,size,refer,
					xData,yData,nPoint,scale);
	_nData++;
};

void GraphData::setNPoint(int nP) {
	for(int i=0; i<_nData; i++) {
		if(!graphSet[i]->isScan()) {
			graphSet[i]->setNPoint(nP);
		}
	}
}

void GraphData::setSummaryData(void) {
	for(int i=0; i<_nData; i++) {
		if(graphSet[i]->isProfile()) {
			graphSet[i]->setData(
				(double*)sum::getdata(pnameX),
				(double*)sum::getdata(pnameX),
				sum::getn());
		} else if(!graphSet[i]->isScan()) {
			graphSet[i]->setData(
				(double*)sum::getdata(pnameX),
				(double*)sum::getdata(graphSet[i]->pname()),
				sum::getn());		 // data
		}
	}
}

void GraphData::clean(void) {
	for(int i=0; i<_nData; i++) {
		delete graphSet[i];
	}
	_nData = 0;
}

bool makePName(QString& pname,
		const char* what, const char* entity) {
	QString prefix;
	if(strcmp(what,"SCAN") == 0) {
		prefix = "S_";
		pname = prefix + entity;
		return true;
	} else { 
		if(strcmp(what,"FLUX") == 0) prefix = "F_";
		else if(strcmp(what,"INDEX") == 0) prefix = "SI_";
		else if(strcmp(what,"DESI") == 0) prefix = "DESI_";
		else if(strcmp(what,"INTEGRAL") == 0) prefix = "";
		pname = prefix + entity;
// Parsing:
		sum::getref(pname.data(), sum::FIRSTMEAN);
		if(nr::erget() == nr::ERNOD) {
			QString mess = nr::ermessage().c_str();
			showStrongMessage(mess);
			nr::erreset();
			pname = "";
			return false;
		}
		return true;
	}
}
  
GraphSet* GraphData::getGraphSet(const char* what, const char* entity) {
	QString pname;
	if(makePName(pname, what, entity)) {
		for(int i=0; i<_nData; i++) {
			if(pname == graphSet[i]->pname()) {
				return graphSet[i];
			} 
		}
	}
	return NULL;
}

GraphData::~GraphData(void) {
	clean();
}

void ShowSet::set(const char* show) {
	QString str(show);
	_graph = false;
	_text = false;
	_err = false;
	if(str.contains('G',false)) _graph = true;
	if(str.contains('F',false)) {
		_text = true;
		_err = true;
		return;
	}
	if(str.contains('B',false)) _text = true;
	return;
}

void ShowSet::set(ShowSet& showSet) {
	_graph = showSet.graph();
	_text = showSet.text();
	_err = showSet.err();
}

OutControl::OutControl(void) {
	_nDSI = 0;
	isFirst = true;
	_graphReady = false;
	_graphReDraw = false;

// Profile:
	_nLut = 0;
	_nZsmooth = 0;
	_zSmooth = NULL;
	_cn2Smooth = NULL;
}

OutControl::~OutControl() {
	for(int i=0; i<_nLut; i++) delete lutList[i];
	if(_zSmooth) delete[]_zSmooth;
	if(_cn2Smooth) delete[]_cn2Smooth;
}

sum::reftype getRefType(QString& refer) {
	sum::reftype rtype = sum::FIRSTMEAN;
	if(refer.upper() == "FIRSTMEAN") rtype = sum::FIRSTMEAN;
	else if(refer.upper() == "LASTMEAN") rtype = sum::LASTMEAN;
	else if(refer.upper() == "SEASONMEAN") rtype = sum::SEASONMEAN;
// lll
	return rtype;
}

bool DisplayData::isgen(void) {
	QString str(_entity);
	if(str.contains("S",false)) return true;
	else return false;
}

bool DisplayData::iscross(void) {
	QString str(_entity);
	int pos = str.find("S",0,false);
	if(pos != -1) str.remove(pos,1);
	return(str.length() == 2); // AB, BC ....=> true
}

sum::reftype readDisplayData(const char* section,
			QArray<DisplayData>& arr) {
	QArray<const char*> entity;
	QString str;
	bool first = true;
	leafDataToStr(str, first, cfgFile,
		"Display", section, "EntityList");
	QString path = "turbina.cfg\\Display\\";
	path += section;
	QString what = section;
	what = what.upper();
	int nEnt = stringToStrArray(&str, entity,
		what.data(),
		path+"\\EntityList",
			"entity");

	QArray<double> refer;
	leafDataToStr(str, first, cfgFile,
		"Display", section, "ReferVal");
	int nRefer = stringToDoubleArray(&str, refer,
		path+"\\ReferVal");

	QArray<const char*> show;
	leafDataToStr(str, first, cfgFile,
		"Display", section, "Show");
	int nShow = stringToStrArray(&str, show,
		what.data(),
		path+"\\Show",
			"show");

	QArray<const char*> color;
	leafDataToStr(str, first, cfgFile,
		"Display", section, "Color");
	int nCol = stringToStrArray(&str, color,
		what.data(),
		path+"\\Color",
			"color");

	QArray<int> size;
	leafDataToStr(str, first, cfgFile,
		"Display", section, "Size");
	int nSize = stringToIntArray(&str, size,
		path+"\\Size");

	QString referType;
	leafDataToStr(referType, first, cfgFile,
		"Display", section, "Reference");
	sum::reftype rtype = getRefType(referType);

	arr.resize(nEnt);
	for(int i=0; i<nEnt; i++) {
		arr[i].setEntity(entity[i]);
		if(i >= nRefer) arr[i].setRefer(sum::NODATA);
		else arr[i].setRefer(refer[i]);
		if(i >= nShow) arr[i].showSet()->set("No");
		else arr[i].showSet()->set(show[i]);
		if(i >= nCol) arr[i].setColor("white");
		else arr[i].setColor(color[i]);
		if(i >= nSize) arr[i].setSize(1);
		else arr[i].setSize(size[i]);
	}
	return rtype;
}

sum::reftype readProfileDisplayData(DisplayData& data) {
	bool first = true;
	QString showName;
	leafDataToStr(showName, first, cfgFile,
		"Display", "Profile", "Show");
	double refer;
	leafDataToDouble(refer, first, cfgFile,
		"Display", "Profile", "ReferVal");
	const char* show = getShow(showName);

	QString typeName;
	leafDataToStr(typeName, first, cfgFile,
		"Display", "Profile", "Type");

	QString colorName;
	leafDataToStr(colorName, first, cfgFile,
		"Display", "Profile", "Color");
	const char* color = getColor(colorName);

	int size;
	leafDataToInt(size, first, cfgFile,
		"Display", "Profile", "Size");

	QString referType;
	leafDataToStr(referType, first, cfgFile,
		"Display", "Profile", "Reference");
	sum::reftype rtype = getRefType(referType);

	data.showSet()->set(show);
	data.setColor(color);
	data.setSize(size);
	data.setRefer(refer);
	return rtype;
}

bool isConfigChanged(void) {
	bool c = false;

#if defined __MOTOR__ || defined __SCANMOTOR__
	c = c || isLeafChanged(cfgFile,
		"Display", "Scan", "EntityList");
	c = c || isLeafChanged(cfgFile,
		"Display", "Scan", "Show");
	c = c || isLeafChanged(cfgFile,
		"Display", "Scan", "Color");
	c = c || isLeafChanged(cfgFile,
		"Display", "Scan", "Size");
	c = c || isLeafChanged(cfgFile,
		"Display", "Scan", "Reference");
	c = c || isLeafChanged(cfgFile,
		"Display", "Scan", "ReferVal");
#endif

	c = c || isLeafChanged(cfgFile,
		"Display", "Flux", "EntityList");
	c = c || isLeafChanged(cfgFile,
		"Display", "Flux", "Show");
	c = c || isLeafChanged(cfgFile,
		"Display", "Flux", "Color");
	c = c || isLeafChanged(cfgFile,
		"Display", "Flux", "Size");
	c = c || isLeafChanged(cfgFile,
		"Display", "Flux", "Reference");
	c = c || isLeafChanged(cfgFile,
		"Display", "Flux", "ReferVal");

	c = c || isLeafChanged(cfgFile,
		"Display", "Index", "EntityList");
	c = c || isLeafChanged(cfgFile,
		"Display", "Index", "Show");
	c = c || isLeafChanged(cfgFile,
		"Display", "Index", "Color");
	c = c || isLeafChanged(cfgFile,
		"Display", "Index", "Size");
	c = c || isLeafChanged(cfgFile,
		"Display", "Index", "Reference");
	c = c || isLeafChanged(cfgFile,
		"Display", "Index", "ReferVal");

	c = c || isLeafChanged(cfgFile,
		"Display", "DESI", "EntityList");
	c = c || isLeafChanged(cfgFile,
		"Display", "DESI", "Show");
	c = c || isLeafChanged(cfgFile,
		"Display", "DESI", "Color");
	c = c || isLeafChanged(cfgFile,
		"Display", "DESI", "Size");
	c = c || isLeafChanged(cfgFile,
		"Display", "DESI", "Reference");
	c = c || isLeafChanged(cfgFile,
		"Display", "DESI", "ReferVal");

	c = c || isLeafChanged(cfgFile,
		"Display", "Integral", "EntityList");
	c = c || isLeafChanged(cfgFile,
		"Display", "Integral", "Show");
	c = c || isLeafChanged(cfgFile,
		"Display", "Integral", "Color");
	c = c || isLeafChanged(cfgFile,
		"Display", "Integral", "Size");
	c = c || isLeafChanged(cfgFile,
		"Display", "Integral", "Reference");
	c = c || isLeafChanged(cfgFile,
		"Display", "Integral", "ReferVal");

	c = c || isLeafChanged(cfgFile,
		"Display", "Profile", "Show");
	c = c || isLeafChanged(cfgFile,
		"Display", "Profile", "Type");
	c = c || isLeafChanged(cfgFile,
		"Display", "Profile", "Color");
	c = c || isLeafChanged(cfgFile,
		"Display", "Profile", "LUT");
	c = c || isLeafChanged(cfgFile,
		"Display", "Profile", "Size");
	c = c || isLeafChanged(cfgFile,
		"Display", "Profile", "Reference");
	c = c || isLeafChanged(cfgFile,
		"Display", "Profile", "Scale");
	c = c || isLeafChanged(cfgFile,
		"Display", "Profile", "ReferVal");
	return c;
}

int toInt255(double val) {
	val *= 255;
	if(val < 0) val = 0;
	else if(val > 255) val = 255;
	return int(val);
}

void OutControl::loadLutTable(const QString& fileName) {
	QFile file(fileName);
	if(!file.open(IO_ReadOnly)) {
		QString messg = "\"" + fileName + "\" not found!";
		throw LoadError(messg);
	}
	QTextStream textStream(&file);
	int nColor = 0;
	int nRow = 0;

	while(!textStream.eof()
			&& (nColor < LUT_LENGTH)) {
		QString line = textStream.readLine();
		nRow++;

		if(line.isEmpty()) continue;
		if(line[0] == '#') continue;

		int n;
		double r,g,b;
		int nval = sscanf(line.data(), "%d%lf%lf%lf", &n, &r,&g,&b);
		if(nval != 4) {
			QString messg;
			messg.sprintf("\"%s\" Row %d: Invalid format!",
				fileName.data(), nRow);
			throw LoadError(messg);
		}
		lutTable[nColor].setRgb(toInt255(r),toInt255(g),toInt255(b));
		nColor++;
	}
	if(nColor != LUT_LENGTH) {
		QString messg = fileName + " is corrupted!";
		throw LoadError(messg);
	}
	
}

const QColor& OutControl::getLutColor(double val) {
	if(val < 0) val = 0;
	else if(val > LUT_LENGTH-1) val = LUT_LENGTH-1;
	return lutTable[int(val)];
}

void OutControl::loadConst(void) {
	jdebug("OutControl::loadConst");
	if(isFirst) {
		double maxAlt;
		leafDataToDouble(maxAlt, isFirst, deviceCfgFile,
			"WeightFunction","MaxAltitude");
		double sqrt2 = sqrt(2);
		_nZsmooth = int(ceil(logN(sqrt2,maxAlt))+2);
		_zSmooth = new double[_nZsmooth];
		_cn2Smooth = new double[_nZsmooth];
		_zSmooth[0] = sqrt2/2;
		for(int i=1; i<_nZsmooth; i++) {
			_zSmooth[i] = _zSmooth[i-1] * sqrt2;
		}

		leafDataToStr(_lutDirName, isFirst, cfgFile, "General",
			"Inputs", "LutDir");
		prepareDirName(_lutDirName);

		QString filter = "*.";
		filter += lutFileExt;
		QDir lutDir(_lutDirName, filter);
		uint nLuts = lutDir.count();
		lutList.resize(nLuts);

		const QFileInfoList *list = lutDir.entryInfoList();
		if(list == 0) {
			printf("LUT-directory is empty!\n");
			exit(1);
		}
		QFileInfoListIterator it(*list);
		QFileInfo *fi;	// pointer for traversing

		_nLut = 0;
	        while((fi=it.current())) {
			QString lutName = fi->fileName();
			lutList[_nLut] = new QString(
				lutName.left(lutName.find(".")));
			_nLut++;
			++it;
		}
	}

	if(!isFirst && !isConfigChanged()) return;

	QString lutFileName;
	leafDataToStr(lutFileName, isFirst, cfgFile,
		"Display", "Profile", "LUT");
	lutFileName.prepend(_lutDirName);
	lutFileName += ".";
	lutFileName += lutFileExt;
	testFileName(lutFileName,1);

	try {
		loadLutTable(lutFileName);
	} catch(LoadError err) {
		showFatalError(err.getMessage());
//		exit(1);
	}

	QString str;
	leafDataToStr(str, isFirst, cfgFile,
			"Display", "Profile", "Type");
	_percentType = false;
	if(str.upper() == "BARS") _percentType = true;

	leafDataToDouble(_cn2ScaleFactor, isFirst, cfgFile,
			"Display", "Profile", "Scale");

	QArray<DisplayData> scan;
	QArray<DisplayData> flux;
	QArray<DisplayData> index;
	QArray<DisplayData> desi;
	QArray<DisplayData> integral;
	DisplayData profile;

#if defined __MOTOR__ || __SCANMOTOR__
	readDisplayData("Scan",scan);
#endif
	_fluxRefType = readDisplayData("Flux",flux);
	_indexRefType = readDisplayData("Index",index);
	_desiRefType = readDisplayData("DESI",desi);
	_integralRefType = readDisplayData("Integral",integral);
	_profileRefType = readProfileDisplayData(profile);

	prepare(scan, flux, index, desi, integral, profile);

	setGraphReady(true,true);

	if(isFirst) {
		connect(tree, SIGNAL(changed()), this, SLOT(change()));
		isFirst = false;
	}
}

void OutControl::change(void) {
		jdebug("OutControl::change");
	loadConst();
}

int chanNumber(const char* chanName) {
	switch(*chanName) {
		case 'A' : return 0;
		case 'B' : return 1;
		case 'C' : return 2;
		case 'D' : return 3;
		default  : return 0;
	}
}

void OutControl::prepare(QArray<DisplayData>& scan,
				QArray<DisplayData>& flux,
				QArray<DisplayData>& index,
				QArray<DisplayData>& desi,
				QArray<DisplayData>& integral,
				DisplayData& profile) {
// Prepare GraphData:

	graphData.clean();

#if defined __MOTOR__ || defined __SCANMOTOR__
	for(uint i=0; i<scan.size(); i++) {
		QString pnameY;
		if(scan[i].showGraph() &&
				makePName(pnameY,"SCAN",scan[i].entity())) {
			graphData.add(pnameY,
				scan[i].color(),
				scan[i].size(),
				1.0, // t.k. refer tut ne imeet smysla
//				sum::FIRSTMEAN,
				scanX,
				scanY[chanNumber(scan[i].entity())],
				scanLength);		 // data
		}
	}
#endif

	for(uint i=0; i<flux.size(); i++) {
		QString pnameY;
		if(flux[i].showGraph() &&
				makePName(pnameY,"FLUX",flux[i].entity())) {
			graphData.add(pnameY,
				flux[i].color(),
				flux[i].size(),
				flux[i].refer());
//				_fluxRefType);
		}
	}

	for(uint i=0; i<index.size(); i++) {
		QString pnameY;
		if(index[i].showGraph() &&
				makePName(pnameY,"INDEX",index[i].entity())) {
			graphData.add(pnameY,
				index[i].color(),
				index[i].size(),
				index[i].refer());
//				_indexRefType);
		}
	}

	for(uint i=0; i<desi.size(); i++) {
		QString pnameY;
		if(desi[i].showGraph() &&
				makePName(pnameY,"DESI",desi[i].entity())) {
			graphData.add(pnameY,
				desi[i].color(),
				desi[i].size(),
				desi[i].refer());
//				_desiRefType);
		}
	}

	for(uint i=0; i<integral.size(); i++) {
		QString pnameY;
		if(integral[i].showGraph() &&
				makePName(pnameY,"INTEGRAL",integral[i].entity())) {
			graphData.add(pnameY,
				integral[i].color(),
				integral[i].size(),
				integral[i].refer());
//				_integralRefType);
		}
	}

	_drawProfile = profile.showGraph();
	if(profile.showGraph()) {
		QString pnameY = "CN2";
		graphData.add(pnameY,
			profile.color(),
			profile.size(),
			profile.refer(),
//			_profileRefType,
			NULL,
			NULL,
			0,
			_cn2ScaleFactor);
	}

	double totalSize = 0;
	for(int i=0; i<graphData.nGraph(); i++) {
		totalSize += graphSet(i)->size();
	}

	for(int i=0; i<graphData.nGraph(); i++) {
		double quantum = graphSet(i)->size() / totalSize;
		graphSet(i)->setQuantum(quantum);
	}

	graphData.setSummaryData();

// Prepare textData:

	_showFlux = false;
	for(uint i=0; i<flux.size(); i++) {
		_showFlux = _showFlux || flux[i].showText();
	}
	_showFluxErr = false;
	for(uint i=0; i<flux.size(); i++) {
		_showFluxErr = _showFluxErr || flux[i].showErr();
	}

	_showIndex = false;
	for(uint i=0; i<index.size(); i++) {
		_showIndex = _showIndex || 
			(!index[i].isgen() && index[i].showText());
	}

	_showIndexErr = false;
	for(uint i=0; i<index.size(); i++) {
		_showIndexErr = _showIndexErr ||
			(!index[i].isgen() && index[i].showErr());
	}

	_showIndexG = false;
	for(uint i=0; i<index.size(); i++) {
		_showIndexG = _showIndexG ||
			(index[i].isgen() && index[i].showText());
	}

	_showIndexGerr = false;
	for(uint i=0; i<index.size(); i++) {
		_showIndexGerr = _showIndexGerr ||
			(index[i].isgen() && index[i].showErr());
	}

	_nDSI = 0;
	for(uint i=0; i<index.size(); i++) {
		if((_nDSI < MAX_DSI_NUMBER) &&
				index[i].iscross() &&
				index[i].showText()) {
			const char* idx = getCrosIdx(index[i].entity());
			bool first = true;
			for(int j=0; j<_nDSI; j++) {
				if(strcmp(idx,dsiName[j]) == 0)
					first = false; 
			}
			if(first) {
				dsiName[_nDSI] = idx;
				_nDSI++;
			}
		}
	}

	_showDSI = false;
	for(uint i=0; i<index.size(); i++) {
		_showDSI = _showDSI ||
			(!index[i].isgen() &&
			index[i].iscross() &&
			index[i].showText());
	}

	_showDSIerr = false;
	for(uint i=0; i<index.size(); i++) {
		_showDSIerr = _showDSIerr ||
			(!index[i].isgen() &&
			index[i].iscross() &&
			index[i].showErr());
	}

	_showDSIg = false;
	for(uint i=0; i<index.size(); i++) {
		_showDSIg = _showDSIg ||
			(index[i].isgen() &&
			index[i].iscross() &&
			index[i].showText());
	}

	_showDSIgErr = false;
	for(uint i=0; i<index.size(); i++) {
		_showDSIgErr = _showDSIgErr ||
			(index[i].isgen() &&
			index[i].iscross() &&
			index[i].showErr());
	}

	_showDESI = false;
	for(uint i=0; i<desi.size(); i++) {
		_showDESI = _showDESI || desi[i].showText();
	}
	_showDESIErr = false;
	for(uint i=0; i<desi.size(); i++) {
		_showDESIErr = _showDESIErr || desi[i].showErr();
	}

	_showSee = false;
	_showHeff = false;
	_showIsopl = false;
	_showM2 = false;
	_showM0 = false;
	_showTau = false;
	for(uint i=0; i<integral.size(); i++) {
		if(!integral[i].showText()) continue;
		QString str = integral[i].entity();
		str = str.upper();
		if((str == "SEE") || (str == "FSEE"))
			_showSee = true;
		else if((str == "HEFF") || (str == "FHEFF"))
			_showHeff = true;
		else if(str == "ISOPL") _showIsopl = true;
		else if(str == "M2") _showM2 = true;
		else if(str == "M0") _showM0 = true;
		else if(str == "TAU") _showTau = true;
	}

	_showProfile = profile.showText();
}

bool OutControl::isGraphReady(void) {
// Tut vozmozhno nesinkhronnoe obrachenie.
	bool ready;

	graphExportMutex.lock();	
	ready = _graphReady;
	graphExportMutex.unlock();

	return(ready);
}

const QString undefined = "Undefined";

const QString& OutControl::getLutName(int nL) {
	if((nL < 0) || (nL >= _nLut)) return undefined;
	else return(*lutList[nL]);
}

bool OutControl::reDraw(void) {
// Tut vozmozhno nesinkhronnoe obrachenie.
	bool redraw;

	graphExportMutex.lock();	
	redraw = _graphReDraw;
	graphExportMutex.unlock();

	return(redraw);
}

void OutControl::setGraphReady(bool ready, bool reDraw) {
// Tut vozmozhno nesinkhronnoe obrachenie.

	graphExportMutex.lock();	
	_graphReady = _graphReady || ready;
	_graphReDraw = _graphReDraw || reDraw;
	graphExportMutex.unlock();

}

void OutControl::resetGraphReady(void) {
// Tut vozmozhno nesinkhronnoe obrachenie.

	graphExportMutex.lock();	
	_graphReady = false;
	_graphReDraw = false;
	graphExportMutex.unlock();

}

void OutControl::addSummaryData(double ut, double mag,
				double airmass,
				bool ready,
				bool isgen) {

	graphExportMutex.lock();
	sum::add(ut, mag, airmass, isgen);
	graphData.setSummaryData();
	_graphReady = ready;
	graphExportMutex.unlock();

}

void OutControl::resetSummaryData(void) {
	graphData.setSummaryData();
}
