#include <math.h>	// To prevent problems with missing built-in fabs():
#include <string.h>
#include "power.h"
#include "tools.h"
#include "names.h"
#include "exchange.h"
#include "cfg.h"

Power::Power(void) {
	jdebug("power::power");
	_name = "HV power";
	_identName = "HV power";
	_identSection = "Common";
}

Power::~Power() {
}

void Power::resetVariables(void) {
	jdebug("Power::resetVariables");
	_safety			= UNDEFINED;
	_voltageValue	= UNDEFINED;
}

int Power::loadOwnConst(void) {
	jdebug("power: LoadOwnConst");
	bool first = true;
	leafDataToChar(_setVoltage,first,deviceCfgFile,_name,"Command","SetVoltage");
	leafDataToChar(_getVoltage,first,deviceCfgFile,_name,"Command","GetVoltage");
	leafDataToChar(_highOn,first,deviceCfgFile,_name,"Command","HighON");
	leafDataToChar(_highOff,first,deviceCfgFile,_name,"Command","HighOFF");
	leafDataToChar(_safetyOn,first,deviceCfgFile,_name,"Command","SafetyON");
	leafDataToChar(_safetyOff,first,deviceCfgFile,_name,"Command","SafetyOFF");
	leafDataToChar(_getTemper,first,deviceCfgFile,_name,"Command","GetTemper");
	if(_setVoltage == 0) return FAIL;
	return SUCCESS;
}

int Power::loadConst(void) {
	jdebug("power: LoadConst");
	if(HardModule::loadOwnConst() == FAIL) return FAIL;
	return loadOwnConst();
}

int Power::setVoltage(double voltage,int dialog,
					                int waitTime) {
	jdebug("power::setVoltage");
	int voltageValue = int(rint(voltage * 0.001 * _const1)) - _const2;
	if(voltageValue < MIN_BYTE_VALUE) voltageValue = MIN_BYTE_VALUE;
	if(voltageValue > MAX_BYTE_VALUE) voltageValue = MAX_BYTE_VALUE;
	return setValue(uchar(voltageValue),_voltageValue,_setVoltage,_getVoltage,
									"Error in setVoltage",waitTime);
}

int Power::getVoltage(void) {
	int hv10;
	if(_const1)
		hv10 = int(rint(100.0*(_voltageValue +_const2) / double(_const1)));
	else hv10 = 0;
	return hv10*10;	// okrugliayem
}

int Power::getTemper(double& temper, int dialog) {
#ifdef __COUNTER_BUTTON__
	int temperValue;
#else
	uchar temperValue;
#endif
	if(getValue(temperValue,_getTemper,dialog) != SUCCESS) return FAIL;
#ifdef __COUNTER_BUTTON__
	temper = -103.0 + double(temperValue) / 4.0;
#else
	temper = (2.5*(double(temperValue)+120.0)-500.0)/10.0;
#endif
	return SUCCESS;
}

int Power::isOverLight(int dialog) {
	uchar status;
	if(getStatus(status,dialog) != SUCCESS) return UNDEFINED;
	if(status & OVR_LIGHT) return YES;
	return NO;
}

int Power::isHVLock(int dialog) {
	uchar status;
	if(getStatus(status,dialog) != SUCCESS) return UNDEFINED;
	if(status & HV_LOCK) return YES;
	return NO;
}

int Power::isHVOn(int dialog) {
	uchar status;
	if(getStatus(status,0) != SUCCESS) return UNDEFINED;
	if(status & HV_ON) return ON;
	return OFF;
}

int Power::testOverLight(int dialog) {
	int lock = isHVLock(dialog);
	if(lock == NO) return SUCCESS;
	if(lock == UNDEFINED) return FAIL;
	jdebug("power: Overlight!!!");
	turnHV(OFF,dialog);
	const char* errText = "Overlight! High Voltage turned off!";
	reportError(_identName,errText,dialog,1,"Continue");
	return FAIL;
}

int Power::turnHV(int state, int dialog) {
	jdebug("turnHV");
	if((state==ON) && _safety) {	//  safety
		turnSafety(OFF, dialog);
		turnSafety(ON, dialog);
    }

	int oldHV = UNDEFINED;	//   hv
	int retval = turn(state,oldHV,_highOn,_highOff,dialog);
	if(retval != SUCCESS) return retval;

	if(state == ON) {		//  
		wait_ms(HV_LOCK_WAIT);
		retval = testOverLight(dialog);
		if(retval != SUCCESS) return retval; 
	}
	return SUCCESS;
}

int Power::turnHV(const char* state, int dialog) {
	if(strcasecmp(state,"ON") == 0) return turnHV(ON, dialog);
	else return turnHV(OFF, dialog);
}

int Power::turnSafety(int state, int dialog) {
	jdebug("turnSafety");
	return turn(state,_safety,_safetyOn,_safetyOff,dialog);
}

int Power::turnSafety(const char* state, int dialog) {
	if(strcasecmp(state,"ON") == 0)  return turnSafety(ON, dialog);
	else return turnSafety(OFF, dialog);
}
