#include <math.h>
#include "scanmotor.h"
#include "tools.h"
#include "names.h"
#include "exchange.h"
#include "cfg.h"

extern FILE * out_file;

ScanMotor::ScanMotor(void) {
	jdebug("ScanMotor::ScanMotor");
	_name = "ScanMotor";
	_identName = "ScanMotor";
	_identSection = "Common";
	resetVariables();
}

ScanMotor::~ScanMotor() {
}

void ScanMotor::resetVariables(void) {
	jdebug("ScanMotor::resetVariables");
	_led			= UNDEFINED;
	_winding		= UNDEFINED;
	_currentSpeed	= UNDEFINED; 
}

int ScanMotor::loadOwnConst(void) {
	jdebug("ScanMotor::LoadOwnConst");
	bool first = true;
	leafDataToChar(_getPosition,first,deviceCfgFile,_name,"Command","GetPosition");
	leafDataToChar(_shiftAt,first,deviceCfgFile,_name,"Command","ShiftAt");
	leafDataToChar(_setSpeed,first,deviceCfgFile,_name,"Command","SetSpeed");
	leafDataToChar(_getSpeed,first,deviceCfgFile,_name,"Command","GetSpeed");
	leafDataToChar(_atLeft,first,deviceCfgFile,_name,"Command","AtLeft");
	leafDataToChar(_atRight,first,deviceCfgFile,_name,"Command","AtRight");
	leafDataToChar(_ledOn,first,deviceCfgFile,_name,"Command","LedOn");
	leafDataToChar(_ledOff,first,deviceCfgFile,_name,"Command","LedOff");
	leafDataToChar(_clearAbs,first,deviceCfgFile,_name,"Command","ClearAbs");

	leafDataToChar(_windingOn, first,deviceCfgFile,_name,"Command","WindingOn");
	leafDataToChar(_windingOff, first,deviceCfgFile,_name,"Command","WindingOff");

	leafDataToChar(_testMotion,first,deviceCfgFile,_name,"Command","TestMotion");
	leafDataToChar(_stopMotion,first,deviceCfgFile,_name,"Command","StopMotion");

	if(_stopMotion==0) return FAIL;
	return SUCCESS;
}

int ScanMotor::loadConst(void) {
	jdebug("ScanMotor::LoadConst");
	if(HardModule::loadOwnConst() == FAIL) return FAIL;
	return loadOwnConst();
}

int ScanMotor::reset(int dialog) {
	_winding = UNDEFINED;
	int ret = HardModule::reset(dialog);
	if(ret == SUCCESS) _winding = ON;
	return ret;
}

int ScanMotor::getPosition(int& position, int dialog) {
	jdebug("ScanMotor::getPosition");
	return getValue(position, _getPosition, dialog);
}

int ScanMotor::setStepPeriod(double stepPeriod, int dialog) {
	jdebug("ScanMotor::setStepPeriod");
//	stepPeriod(ms) > 0 !
	if(stepPeriod < 0) stepPeriod = 0;
	int speed = int(rint(stepPeriod*(_const3+(_const4<<8)-1.0)/8.0));
	return setValue(speed, _currentSpeed, _setSpeed, _getSpeed,
								"Error in setSpeed!", dialog);
}

int ScanMotor::waitMotor(int dialog) {
	for(int i=0; i<MAX_MOTOR_TEST_INTERVALS; i++) {
		int retval = isMotion(dialog);		
		switch(retval) {
			case UNDEFINED	: return FAIL;
			case YES		: break;
			case NO			: 
					// Posle uspeshoi ustanovki posizii ne zhdiom, t.k. scanmotor bezinercionnyj pochti:
					//			wait_ms(APERTURE_DELAY);
								return SUCCESS;
        }
        wait_ms(DEVICE_WAIT_INTERVAL);
    }
    stopMotion(dialog);
    const char* errText = "Infinite Motion!";
    reportError(_identName,errText,dialog);
    return FAIL;
}

int ScanMotor::shiftAt(int shift, int dialog) {
	jdebug("ScanMotor::shiftAt %d",shift);
// shift <> 0
	if(sendValue(_shiftAt, shift, dialog) != SUCCESS) return FAIL;
	return SUCCESS;
}

const int gap = 16;

int ScanMotor::atLeft(int dialog) {
	jdebug("ScanMotor::atLeft");
	uchar status;
	if(getStatus(status,dialog) != SUCCESS) return FAIL;
	if(status & LEFT_STOP) {
		printf("Left stop is achieved!\n");
		if(shiftAt(gap, dialog) != SUCCESS) return FAIL;
		if(waitMotor(dialog) != SUCCESS) return FAIL;
	}
	if(sendCommand(_atLeft, dialog) != SUCCESS) return FAIL;
	return waitMotor(dialog);
}

int ScanMotor::atRight(int dialog) {
	jdebug("ScanMotor::atRight");
	uchar status;
	if(getStatus(status,dialog) != SUCCESS) return FAIL;
	if(status & RIGHT_STOP) {
		printf("Right stop is achieved!\n");
		if(shiftAt(-1*gap, dialog) != SUCCESS) return FAIL;
		if(waitMotor(dialog) != SUCCESS) return FAIL;
	}
	if(sendCommand(_atRight, dialog) != SUCCESS) return FAIL;
	return waitMotor(dialog);
}

int ScanMotor::clearAbs(int dialog) {
	jdebug("ScanMotor::clearAbs");
	return sendCommand(_clearAbs, dialog);
}

int ScanMotor::turnLed(int state, int dialog) {
	jdebug("ScanMotor::turnLed");
	return turn(state, _led, _ledOn, _ledOff, dialog);
}

int ScanMotor::turnWinding(int state, int dialog) {
	jdebug("ScanMotor::turnWinding");
	return turn(state, _winding, _windingOn, _windingOff, dialog);
}
