/*--------------------------------------------------------------------------*/
/** @file scanmotor.h
@brief  Implementation of interaction (command calls and replies from modules, 
their current data and status keeping, etc.) with stepper motor.

                  MASS project: TURBINA module file
                       header file for scanmotor.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef SCANMOTOR_H
#define SCANMOTOR_H

#include "module.h"

#define MAX_MOTOR_TEST_INTERVALS	500 //	2.5 sec if dev_wait_int = 5ms
#define MAX_SYNCH_SHIFT				386
#define MIN_SCAN_POSITION			0
#define MAX_SCAN_POSITION			(MAX_SYNCH_SHIFT-1)

// Status of motor:

#define	NEW_STEP	0x01	// new microstep is done
#define	LEFT_STOP	0x02	// left stop is achieved
#define	RIGHT_STOP	0x04	// right stop is achieved
#define	FORWARD		0x40	// forvard motion
#define	MOTION		0x80	// motor is moving

class ScanMotor : virtual public HardModule {

private:

	uchar _getPosition;
	uchar _shiftAt;
	uchar _setSpeed;
	uchar _getSpeed;
	uchar _atLeft;
	uchar _atRight;
	uchar _ledOn;
	uchar _ledOff;
	uchar _clearAbs;
	uchar _windingOn;
	uchar _windingOff;

	int _led;	// ON or OFF
	int _winding;	// ON or OFF
	int _currentSpeed;

public:

	ScanMotor(void);
	virtual ~ScanMotor();

	virtual void resetVariables(void);
	virtual int loadOwnConst(void);
	virtual int loadConst(void);

	int reset(int dialog=1);
	int getPosition(int& position, int dialog=1);
	int setStepPeriod(double stepPeriod, int dialog=1);
	int waitMotor(int dialog);
	int shiftAt(int shift, int dialog=1);
	int atLeft(int dialog=1);
	int atRight(int dialog=1);
	int clearAbs(int dialog=1);
	int turnLed(int state, int dialog=1);
	int turnWinding(int state, int dialog=1);
};

#endif
