#include <stdio.h>
#include "svexchange.h"
#include "trbsocket.h"
#include "names.h"
#include "success.h"
#include "trberror.h"

using namespace std;

namespace SVexchange {
	status _status = PARKED;
	status _erstatus = PARKED;
//	string _newObjectID = "";
	string _data = "";
}

int readInstruction(string& from, int& comid, string& keyword,
							string& param, string& paramData) {
	comid = -1;
	keyword = "";
	param = "";
	paramData = "";
//	char* icopy = strdup( string.c_str());	
//  char* ptok = strtok( icopy, " \t");

//printf("line=%s\n",from.c_str());
	// remove possible leading spaces:
	string::size_type i_comid;
	while (!(i_comid = from.find(" ",0))) from.erase(0,1);

//printf("ltrimmed line=%s\n",from.c_str());

	if(i_comid == string::npos) return FAIL;
	string comid_str = from.substr(0,i_comid);
	comid = atoi(comid_str.c_str());
//printf("comid=%d\n",comid);
	if(comid < 0) return FAIL;

	// remove COMID string:
	from = from.erase(0,i_comid);
//printf("removed-comid-line=%s\n",from.c_str());

	// remove possible leading spaces again:
	string::size_type i1;
	while (!(i1 = from.find(" ",0))) from.erase(0,1);
//printf("ltrimmed r-c-line=%s\n",from.c_str());

	if(i1 == string::npos) {
		keyword = from;
//printf("solely keyword=%s\n",keyword.c_str());
		return SUCCESS;
	}
	keyword = from.substr(0,i1);
//printf("keyword=%s.\n",keyword.c_str());

	// remove KEYWORD string:
	from = from.erase(0,i1);
//printf("removed-keyword-line=%s\n",from.c_str());

	// remove possible leading spaces again:
	string::size_type i2;
	while (!(i2 = from.find(" ",0))) from.erase(0,1);
//printf("ltrimmed r-k-line=%s\n",from.c_str());

	i2 = from.find("=",0);
	if(i2 == string::npos) {
		i2 = from.find(" ",0);
		if(i2 == string::npos) {
			param = from;
		} else {
			param = from.substr(0,i2);
		}
//printf("parameter-only=%s\n",param.c_str());
		return SUCCESS;
	}

	param = from.substr(0,i2);
//printf("parameter=%s\n",param.c_str());
	string::size_type i3 = from.find(" ",i2+1);
	if(i3 == string::npos) {
		paramData = from.substr(i2+1);
	} else {
		paramData = from.substr(i2+1,i3-i2-1);
	}
//printf("paramData=%s\n",paramData.c_str());
	return SUCCESS;
}

int SVexchange::translate(std::string& what, Instruction& instruct) {
	string command = "";
	string param = "";
	string value = "";
	int comid;
	if(readInstruction(what, comid, command, param, value) == FAIL) {
		setErstatus(ERSYN);
		sendError(comid);
		return FAIL;
	}

	instruct.comid = comid;

	if( strcasecmp(     command.c_str(), "GET")  == 0 ) instruct.command = GET;
	else if( strcasecmp(command.c_str(), "SET")  == 0 ) instruct.command = SET;
	else if( strcasecmp(command.c_str(), "INIT") == 0 ) instruct.command = INIT;
	else if( strcasecmp(command.c_str(), "PARK") == 0 ) instruct.command = PARK;
    else if( strcasecmp(command.c_str(), "QUIT") == 0 ) instruct.command = QUIT;
	else if( strcasecmp(command.c_str(), "RUN")  == 0 ) instruct.command = RUN;
	else if( strcasecmp(command.c_str(), "STOP") == 0 ) instruct.command = STOP;
	else instruct.command = UNKNOWN_COMMAND;

	if( strcasecmp( param.c_str(), "IDENT") == 0  ) instruct.param = IDENT;
	else if( strcasecmp( param.c_str(), "STATUS") == 0 ) instruct.param = STATUS;
//	else if((param == "DATA") || (param == "data")) instruct.param = DATA;
	else if( strcasecmp( param.c_str(), "OBJECT") == 0 ) instruct.param = OBJECT;
    else if( strcasecmp( param.c_str(), "SCEN1") == 0 ) instruct.param = SCEN1;
    else if( strcasecmp( param.c_str(), "SCEN2") == 0 ) instruct.param = SCEN2;
	else if( strcasecmp( param.c_str(), "BKGR") == 0 ) instruct.param = BKGR;
	else if( strcasecmp( param.c_str(), "FLUX") == 0 ) instruct.param = FLUX;
	else if( strcasecmp( param.c_str(), "INTEGRAL") == 0 ) instruct.param = INTEGRAL;
	else if( strcasecmp( param.c_str(), "SCIND") == 0 ) instruct.param = SCIND;
    else if( strcasecmp( param.c_str(), "CENTER") == 0 ) instruct.param = CENTER;
	else if( strcasecmp( param.c_str(), "XPROFILE") == 0 ) instruct.param = XPROFILE;
	else if( strcasecmp( param.c_str(), "LPROFILE") == 0 ) instruct.param = LPROFILE;
	else if( strcasecmp( param.c_str(), "NOW") == 0 ) instruct.param = NOW;
	else if( param == "") instruct.param = NOTHING;
	else instruct.param = UNKNOWN_PARAM;
	
	instruct.value = value;
	return SUCCESS;
}

void SVexchange::setStatus(status stat) {
	_status = stat;
}

void SVexchange::setErstatus(status erstatus) {
	_erstatus = erstatus;
}

void SVexchange::setData(const char* data) {
	_data = data;
}

const char* typeStatus(SVexchange::status stat) {
	switch(stat) {
		case SVexchange::READY	: return "READY";
		case SVexchange::PARKED	: return "PARKED";
		case SVexchange::LOCAL	: return "LOCAL";
		case SVexchange::BUSY	: return "BUSY";
		case SVexchange::ERSYN	: return "ERSYN";
		case SVexchange::ERFAT	: return "ERFAT";
		case SVexchange::ERANG	: return "ERANG";
		case SVexchange::NOSTAR	: return "NOSTAR";
		default		: return "UNKNOWN";
	}
}

char toSV[300];

int SVexchange::sendOK(int comid,int waitsec) {
	if(waitsec) {
//VK		sprintf(toSV,"OK status=%s wait=%d\n",typeStatus(_status),waitsec);
		sprintf(toSV,"OK wait=%d\n", waitsec);
	} else {
		sprintf(toSV,"OK status=%s\n",typeStatus(_status));
	}
	return trbsocket::sendToSV(comid, toSV);
}

int SVexchange::sendOK(int comid, std::string& result) {
	sprintf(toSV,"OK %s status=%s\n",result.c_str(), typeStatus(_status));
	return trbsocket::sendToSV(comid, toSV);
}

int SVexchange::sendIdent(int comid) {
	sprintf(toSV,"OK ident=\"TURBINA V%s\"\n",VERSION);
	return trbsocket::sendToSV(comid, toSV);
}

int SVexchange::sendStatus(int comid) {
	sprintf(toSV,"OK status=%s\n",typeStatus(_status));
	return trbsocket::sendToSV(comid, toSV);
}

int SVexchange::sendData(int comid, const char* mode, const char* result ) {
//VK
	sprintf(toSV,"OK %s=\"%s\"\n", mode, result );
	return trbsocket::sendToSV(comid, toSV);
}

int SVexchange::sendError(int comid) {
	sprintf(toSV,"ERROR status=%s\n",typeStatus(_erstatus));
	return trbsocket::sendToSV(comid, toSV);
}

