//#define JDEBUG 1
#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>	// usleep
#include <qstring.h>
#include <qmessagebox.h>
#include <math.h>
#include "exchange.h"
#include "trberror.h"
#include "tools.h"
#include "trbconst.h"
#include "messages.h"

void wait_ms(int msec) {
	if(msec > S2MS) {
		QString str_add;
		QString str = "Wait " +
			str_add.setNum(msec/S2MS) + " sec...";
		showMessage(str, msec);
	}
	ulong mksec = ulong(msec*MS2MKS);
	usleep(mksec);
}

void reportError(const char* name, const char* errText, int dialog, 
							int nbuttons,		// nbuttons = 1 or 2
							const char* ignore,
							const char* abort) {
	QString lastError;
	lastError.sprintf("\"%s\": %s",name,errText);
	exch::setLastErrorText(lastError);
	if(!dialog) return;
	if(trberr::quitProgram()) return;
	return;
	switch (showWarning(lastError, nbuttons, ignore, abort)) {
		case  QMessageBox::Abort : trberr::setCode(TRB_EXIT);
		case  QMessageBox::Ignore: return;
	}
}

// ----------------- without QT -----------------------

double toLogScale(double val) {
	if(val == 0) return 0;
	else return log(val) / log(2) + 1;
}

double fromLogScale(double logVal) {
	if(logVal == 0) return 0;
	else return pow(2,logVal-1);
}

double log2(double val) {
	if(val == 0) return 0;
	return log(val) / log(2);
}

double logN(double val1, double val2) {
	if(val2 == 0) return 0;
	return log(val2) / log(val1);
}

/**
	@brief  Return the concatenation of a character argument
	and QString
	@param  char    leading string (normally, a single character
	@param  string  string to append
	@return C-string as char.arg+Qstring (normally, the stamp for
	writing in mass-file 
*/ 

const char* getStamp(const char * prefix, QDateTime dateTime,
		const char* ext) {
	static QString s;
/*
	int hour = time.hour();
	if((hour+trb_tm::tmZone()) // local time
			< HOUR_DATE_CHANG) {
		hour += HOUR_IN_DAY;
	}
	s.sprintf("%02d:%02d:%02d",
		hour,time.minute(),time.second());
*/
	QDate date = dateTime.date();
	QTime time = dateTime.time();
	s.sprintf("%04d-%02d-%02d %02d:%02d:%02d",
		date.year(), date.month(), date.day(),
		time.hour(), time.minute(), time.second());
	s.prepend(prefix);
	if(ext) s += ext;
	return s.data();  
} 

void jdebug(const char* fmt, ...) {
#ifdef JDEBUG
	QString form = fmt;
	va_list ap;
	va_start(ap, fmt);
	int narg = form.contains("%");

	int posBeg = 0;
	int posEnd = form.find("%",posBeg);
	int length;
	if(posEnd < 0) length = form.length() - posBeg;
	else length = posEnd-posBeg;

	QString form1;
	if(length > 0) {
		form1 = form.mid(posBeg,length);
		printf("%s",form1.data());
	}

	for(int i=0; i<narg; i++) {
		posBeg = posEnd;
		posEnd = form.find("%",posBeg+1);
		if(posEnd < 0) length = form.length() - posBeg;
		else length = posEnd-posBeg;

		form1 = form.mid(posBeg,length);
		char argtype = form.ref(posBeg+1).latin1();

		switch(argtype) {
			case 's': {	// string
				char *s = va_arg(ap, char *);
				printf(form1.data(), s);
				}
				break;
			case 'd': {	// int
				int d = va_arg(ap, int);
				printf(form1.data(), d);
				}
				break;
			case 'f': {	// double
				double f = va_arg(ap, double);
				printf(form1.data(), f);
				}
				break;
			case 'c': {	// char
				char c = (char) va_arg(ap, int);
				printf(form1.data(), c);
				}
				break;
			default: printf("invalid format %c in jdebug: %s",
					argtype,fmt);
		}
	}
	printf("\n");
	va_end(ap);
#endif
}
