#include <math.h>	// To prevent problems with missing built-in fabs():
#include "tracker.h"
#include "tools.h"
#include "names.h"
#include "exchange.h"
#include "cfg.h"

Tracker::Tracker(void) {
	jdebug("tracker::tracker");
	_name = "Tracker";
	_identName = "Tracker";
	_identSection = "Common";
	resetVariables();
}

Tracker::~Tracker() {
}

int Tracker::getConst(void) {
	jdebug("Tracker::getConst");
	int retval = HardModule::getConst();
	if(retval != SUCCESS) return FAIL;
	_maxShift = 0;
	if(_const1 != 0) {
		double shft1 = fabs(double(MAX_SIGN_BYTE_VALUE) / double(_const1));
		double shft2 = fabs(double(MIN_SIGN_BYTE_VALUE) / double(_const1));
		_maxShift = (shft1 < shft2) ? shft1 : shft2;
	} else retval = FAIL;
	return retval;
}

void Tracker::resetVariables(void) {
	jdebug("tracker::resetVariables");
}

int Tracker::loadOwnConst(void) {
	jdebug("tracker: LoadOwnConst");
	bool first = true;
	leafDataToChar(_shiftRA,first,deviceCfgFile,_name,"Command","ShiftRA");
	leafDataToChar(_shiftDec,first,deviceCfgFile,_name,"Command","ShiftDec");
	leafDataToChar(_moveRA,first,deviceCfgFile,_name,"Command","MoveRA");
	leafDataToChar(_moveDec,first,deviceCfgFile,_name,"Command","MoveDec");
	leafDataToChar(_forceMotion,first,deviceCfgFile,_name,"Command","ForceMotion");
	if(!_forceMotion) _forceMotion    = 0x83;
	leafDataToChar(_stopMotion,first,deviceCfgFile,_name,"Command","StopMotion");
	leafDataToChar(_testMotion,first,deviceCfgFile,_name,"Command","TestMotion");
	if(_shiftRA == 0) return FAIL;
	return SUCCESS;
}

int Tracker::loadConst(void) {
	jdebug("tracker: LoadConst");
	if(HardModule::loadOwnConst() == FAIL) return FAIL;
	return loadOwnConst();
}

char Tracker::toSygnByteShift(double shift) {
	double val = shift * _const1;
	if(val > MAX_SIGN_BYTE_VALUE) val = MAX_SIGN_BYTE_VALUE;
	else if(val < MIN_SIGN_BYTE_VALUE) val = MIN_SIGN_BYTE_VALUE;
	return char(val);
}

int Tracker::shift(int axis, double shift, int dialog) {
	jdebug("tracker::shift");
	while(fabs(shift) != 0) {
		double small_shift = shift;
		if(fabs(shift) > _maxShift) {
			small_shift = shift/fabs(shift)*_maxShift;
		}
		int retval = smallShift(axis, small_shift, dialog);
		if(retval != SUCCESS) return retval;
		shift -= small_shift;
	}
	return SUCCESS;
}

int Tracker::smallShift(int axis, double shift, int dialog) {
	jdebug("tracker::smallShift");
	char shift_char = toSygnByteShift(shift);
	uchar shiftCommand;
	if(axis == RA) shiftCommand = _shiftRA;
	else shiftCommand = _shiftDec;
	
	if(sendValue(shiftCommand, (uchar)shift_char, dialog) != SUCCESS) return FAIL;

	for(int i=0; i<MAX_TRACKER_TEST_INTERVALS; i++) {
		int retval = isMotion(dialog);
		switch(retval) {
			case UNDEFINED	: return FAIL;
			case YES		: break;
			case NO			: return SUCCESS;
		}
		wait_ms(DEVICE_WAIT_INTERVAL);
	}
	stopMotion(dialog);
	const char* errText = "Infinite Motion!";
	reportError(_identName,errText,dialog);
	return FAIL;
}

int Tracker::move(int axis, uchar direction, int dialog) {
// "Beskonechnoe dvizhenie"
	jdebug("tracker::move");
	if(axis == RA) return sendValue(_moveRA, direction, dialog);
	return sendValue(_moveDec, direction, dialog);
}

int Tracker::forceMotion(int dialog) {
	jdebug("tracker::forceMotion");
	return sendCommand(_forceMotion, dialog);
}
