#include <pthread.h>
#include <unistd.h>
#include <math.h>
#include "trbsv.h"
#include "cfg.h"
#include "tools.h"
#include "trbexport.h"
#include "trberror.h"
#include "svexchange.h"
#include "scenariothr.h"
#include "device.h"
#include "star.h"
#include "success.h"
#include "trbconst.h"

extern DevManager devmanager;
extern Star	star;
extern bool	withDevice;

namespace trbsv {
	bool cycle_condition = true;
	double _pollingTime = 0.1;
	pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
}

void trbsv::loadConst(bool isFirst) {
	jdebug("trbsv::loadConst()");
	leafDataToDouble(_pollingTime, isFirst, cfgFile,
			"Operations","Common","SuperVisorPollingTime");
	if(_pollingTime == 0) _pollingTime=0.1;	// 0.1 sec
}

void trbsv::processSV(void) {

	std::string instruction_string;
	bool dialog = trberr::dialog();
	trberr::enableDialog(false);

	while (SVexport::get(instruction_string) == SUCCESS) {
        SVexchange::Instruction instruct;
		if(SVexchange::translate(instruction_string, instruct) != SUCCESS) continue;

		if(!InfoExport::withSV) {
            SVexchange::setErstatus(SVexchange::LOCAL);
            SVexchange::sendError(instruct.comid);
			continue;
		}

		try {
		switch(instruct.command) {
			case SVexchange::GET	:
				switch(instruct.param) {
					case SVexchange::IDENT :
						SVexchange::sendIdent(instruct.comid);	
						break;

					case SVexchange::STATUS :
						SVexchange::sendStatus(instruct.comid);	
						break;

//VK				case SVexchange::DATA :
//						SVexchange::sendData(instruct.comid);	
//						break;

                    case SVexchange::BKGR :
                        SVexchange::sendData(instruct.comid, "bkgr", devmanager.getBackground());
                        break;

                    case SVexchange::FLUX :
					    SVexchange::sendData(instruct.comid, "flux", devmanager.getFluxes());
                        break;

                    case SVexchange::INTEGRAL :
					    SVexchange::sendData(instruct.comid, "integral", devmanager.getIntegral());
                        break;

                    case SVexchange::SCIND :
					    SVexchange::sendData(instruct.comid, "scind", devmanager.getScind());
                        break;

                    case SVexchange::XPROFILE :
					    SVexchange::sendData(instruct.comid, "xprofile", devmanager.getXProfile());
                        break;

                    case SVexchange::LPROFILE :
					    SVexchange::sendData(instruct.comid, "lprofile", devmanager.getLProfile());
                        break;

                    case SVexchange::CENTER :
                        SVexchange::sendData(instruct.comid, "center", devmanager.getCentering());
                        break;

                    case SVexchange::SCEN1  :
                        SVexchange::sendData(instruct.comid, "scen1", 
									devmanager.getScenario(trbscenario::SV_SC1_SCENARIO));
                        break;

                    case SVexchange::SCEN2  :
                        SVexchange::sendData(instruct.comid, "scen2",
                            		devmanager.getScenario(trbscenario::SV_SC2_SCENARIO));
                         break;
					
					default	:
						throw SVError("");
				};
				break;

			case SVexchange::QUIT :
				if(!devmanager.isCounting()) {
					devmanager.quitProgram();
				} else {
					SVexchange::setErstatus(SVexchange::BUSY);
					SVexchange::sendError(instruct.comid);
				}

				break;

			case SVexchange::INIT :
		
				if(!withDevice) {
					SVexchange::setErstatus(SVexchange::ERFAT);
					SVexchange::sendError(instruct.comid);
					break;
				} 
//VK
				if( !StateExport::isParked()) {
                    SVexchange::setStatus(SVexchange::READY);
                    SVexchange::sendOK(instruct.comid,0);
					break;
				}
				if(devmanager.isCounting()) {
					SVexchange::setErstatus(SVexchange::BUSY);
					SVexchange::sendError(instruct.comid);
				} else {
					SVexchange::setStatus(SVexchange::BUSY);
//					SVexchange::sendOK(instruct.comid,INIT_TMOUT);
					if(devmanager.init() != SUCCESS) {
						SVexchange::setErstatus(SVexchange::ERFAT);
						SVexchange::setStatus(SVexchange::ERFAT);
						SVexchange::sendError(instruct.comid);
    				} else if( devmanager.playInitialScenario() ) {
						devmanager.setCurrentScenario(trbscenario::SV_INIT_SCENARIO);
						devmanager.setPlayScenarioSVcomid(instruct.comid);
						scenariothr::playScenario();
					} else {
	                    SVexchange::setStatus(SVexchange::READY);
    	                SVexchange::sendOK(instruct.comid,0);
					}
				}
				break;

			case SVexchange::PARK :
				if(!withDevice) {
					SVexchange::setErstatus(SVexchange::ERFAT);
					SVexchange::sendError(instruct.comid);
					break;
				}
                if( StateExport::isParked()) {
                    SVexchange::setStatus(SVexchange::PARKED);
                    SVexchange::sendOK(instruct.comid,0);
					break;
                }
				if(devmanager.isCounting()) {
					SVexchange::setErstatus(SVexchange::BUSY);
					SVexchange::sendError(instruct.comid);
				} else {
					SVexchange::setStatus(SVexchange::BUSY);
					SVexchange::sendOK(instruct.comid,PARK_TMOUT);
					devmanager.park();
					SVexchange::setStatus(SVexchange::PARKED);
					SVexchange::sendOK(instruct.comid,0);		//PARKED
				}
				break;

			case SVexchange::RUN :
//VK
                if( StateExport::isParked()) {
                    SVexchange::setErstatus(SVexchange::PARKED);
                    SVexchange::sendError(instruct.comid);
					break;
                }
                if(!star.isStarSelected()) {
                    SVexchange::setErstatus(SVexchange::NOSTAR);
                    SVexchange::sendError(instruct.comid);
                    break;
                }
                if(withDevice && !devmanager.isCounting()) {
					switch(instruct.param) {
						case SVexchange::NOTHING	:
							devmanager.setCurrentScenario(trbscenario::SV_RUN_SCENARIO);
							break;

						case SVexchange::SCEN1		:
							devmanager.setCurrentScenario(trbscenario::SV_SC1_SCENARIO);
							break;

						case SVexchange::SCEN2		:
							devmanager.setCurrentScenario(trbscenario::SV_SC2_SCENARIO);
							break;

						default	:
						throw SVError("");
					}
					devmanager.setPlayScenarioSVcomid(instruct.comid);
                    //_playScenarioSVcomid -?? where is used
					SVexchange::setStatus(SVexchange::BUSY);
					scenariothr::playScenario();
                } else {
                    SVexchange::setErstatus(SVexchange::BUSY);
                    SVexchange::sendError(instruct.comid);
                }
				break;

			case SVexchange::STOP :
				switch(instruct.param) {
					case SVexchange::NOTHING	:
						if(withDevice && devmanager.isCounting()) {
							devmanager.setStopScenarioSVcomid(instruct.comid);
							SVexchange::sendOK(instruct.comid,int(rint(devmanager.getRemainTimeMode())));
							devmanager.stopScenario();
						} else SVexchange::sendOK(instruct.comid,0);
						break;

					case SVexchange::NOW	:
						if(withDevice && devmanager.isCounting()) {
							devmanager.setStopCountingSVcomid(instruct.comid);
							SVexchange::sendOK(instruct.comid,int(rint(devmanager.getCurrentMaxBaseTime())));
							devmanager.stopCounting();
						} else SVexchange::sendOK(instruct.comid,0);
						break;

					default	:
						throw SVError("");
				};
				break;

			case SVexchange::SET :
//VK
                if(devmanager.isCounting()) {
                    SVexchange::setErstatus(SVexchange::BUSY);
                    SVexchange::sendError(instruct.comid);
					break;
                } 

				switch(instruct.param) {
					case SVexchange::OBJECT	:
     					if(star.setCurrentStar(QString(instruct.value.c_str())) == SUCCESS) {
							SVexchange::sendOK(instruct.comid,0);
						} else {
							SVexchange::setErstatus(SVexchange::ERANG);
							SVexchange::sendError(instruct.comid);
	    				}
						break;

                    case SVexchange::SCEN1 :
						if(devmanager.setScenario(trbscenario::SV_SC1_SCENARIO, instruct.value.c_str()) == SUCCESS ){ 
							SVexchange::sendOK(instruct.comid,0);
						} else {
							SVexchange::setErstatus(SVexchange::ERANG);
							SVexchange::sendError(instruct.comid);
	    				}
						break;

                    case SVexchange::SCEN2 :
						if(devmanager.setScenario(trbscenario::SV_SC2_SCENARIO, instruct.value.c_str()) == SUCCESS ){ 
							SVexchange::sendOK(instruct.comid,0);
						} else {
							SVexchange::setErstatus(SVexchange::ERANG);
							SVexchange::sendError(instruct.comid);
	    				}
						break;

					default	:
						throw SVError("");
				};
				break;

			default	:
				throw SVError("");
		}
		} catch (SVError) {
				SVexchange::setErstatus(SVexchange::ERSYN);
				SVexchange::sendError(instruct.comid);
		}
	}
	trberr::enableDialog(dialog);
}

int trbsv::sv_cycle(void) {		// execution sv-command

	while(getCycleCondition()) {

		processSV();

		usleep(ulong(SEC2USEC * _pollingTime));
//		jdebug("sv_cycle:next cycle");
	}
//	printf("trbsv::quit sv_cycle\n");
	return 1;
}

void trbsv::setCycleCondition(bool condition) {
	pthread_mutex_lock(&mutex);
	cycle_condition = condition;
	pthread_mutex_unlock(&mutex);
}

bool trbsv::getCycleCondition(void) {
	pthread_mutex_lock(&mutex);
	bool condition = cycle_condition;
	pthread_mutex_unlock(&mutex);
	return condition;
}

void trbsv::quitSVcycle(void) {
	jdebug("trbsv::quitSVcycle()");

	setCycleCondition(false);

	jdebug("end of trbsv::quitSVcycle()");
}
