#include <qapplication.h>
#include <qfile.h>
#include <qdatetime.h>
#include "turgraph.h"
#include "atmos.hxx"
#include "weif.hxx"

int main( int argc, char ** argv ) {
	fprintf(stdout,"                   TURGRAPH ver.1.1\n");
	QApplication qapp( argc, argv );

	GraphWindow gw( 0,"GraphWindow" );
	qapp.setMainWidget( &gw );
	gw.show();
//	gw.drawAll();
	return qapp.exec();
}



const int pos_met = 24;		//method (L or X)
char met = 'L';

double toLogScale(double val) {
	if(val == 0) return 0;
 	else return log(val) / log(2) + 1;
}

double fromLogScale(double logVal) {
	if(logVal == 0) return 0;
	else return pow(2,logVal-1);
}

double log2(double val) {
	if(val == 0) return 0;
	return log(val) / log(2);
}

double logN(double val1, double val2) {
	if(val2 == 0) return 0;
	return log(val2) / log(val1);
}


const int max_nz = 10;
int nz=0, n_ut=0, max_n_ut=0;
double** z=NULL;
double** cn2=NULL;
double* ut=NULL;
int nZsmooth = 0;
double* zSmooth = NULL;
double* cn2Smooth = NULL;
const double maxAlt = 20;

const int UTLEN = 8;
const int DATEUTLEN = 10+1+UTLEN;

int loadProfile(const char* fname, double t1, double t2) {
	QString fileName(fname);

	QFile file(fileName);
	if(!file.open(IO_ReadOnly)) {
		QString messg = "\"" + fileName + "\" not found!";
		printf("%s\n",messg.data());
		exit(1);
//		throw LoadError(messg);
	}
	QTextStream textStream(&file);

	max_n_ut = 0;

	while(!textStream.eof()) {
		QString line = textStream.readLine();
		if(line.isEmpty()) continue;
		if(line[0] == 'T') {
			if(line[pos_met] == met) {
				max_n_ut++;
			}
		}
		continue;
	}

	file.close();
	file.open(IO_ReadOnly);
	ut = new double[max_n_ut];
	z = new (double*)[max_n_ut];
	cn2 = new (double*)[max_n_ut];
	for(int i=0; i<max_n_ut; i++) {
		z[i] = new double[max_nz];
		cn2[i] = new double[max_nz];
	}
	n_ut = 0;
	int nRow = 0;
	QDate qdate_start;
	bool start = true;
	while(!textStream.eof()) {
		QString line = textStream.readLine();
		nRow++;

		if(line.isEmpty()) continue;
		if(line[0] != 'T') continue;
		if(line[pos_met] != met) continue;

		double d;
		int h=99,m=99,s=99;
	// version 1.6: date components:
		int yy=0,mm=0,dd=0;
	// first try to extract full date, if fails - the time only:
		int shft=0;
		if (sscanf(line.data()+2,"%d-%d-%d %d:%d:%d",&yy,&mm,&dd,&h,&m,&s)!=6) {
			if (sscanf(line.data()+2,"%d:%d:%d",&h,&m,&s)!=3) continue;
			shft = UTLEN; // UT record skipped
		} else {
		// validate date:
			if (yy<2001 || yy>2100 || mm<1 || mm>12 || dd<1 || dd>31) continue;
			shft = DATEUTLEN; // DATE+UT record skipped
		}
	// validate time: we allow m=60 and s=60 (trbtime.cpp bug)
		if (h<0 || h>=100 || m<0 || m>60 || s<0 || s>60) continue;

		double ut_ = ( double(s)/60.0 + double(m) )/60.0 + h;
		if(start) {
			start = false;
			qdate_start.setYMD(yy,mm,dd);
		} else {
			QDate qdate (yy,mm,dd);
			ut_+= qdate_start.daysTo(qdate)*24;
		}
		if( (ut_<t1) || ((t2>0) && (ut_>t2)) ) continue;
		int nval = sscanf(line.data()+shft+2+4, "%d %lf %lf\
		%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf",
		&nz, &d, &d,
		&z[n_ut][0], &cn2[n_ut][0],
		&z[n_ut][1], &cn2[n_ut][1],
		&z[n_ut][2], &cn2[n_ut][2],
		&z[n_ut][3], &cn2[n_ut][3],
		&z[n_ut][4], &cn2[n_ut][4],
		&z[n_ut][5], &cn2[n_ut][5],
		&z[n_ut][6], &cn2[n_ut][6],
		&z[n_ut][7], &cn2[n_ut][7],
		&z[n_ut][8], &cn2[n_ut][8],
		&z[n_ut][9], &cn2[n_ut][9]);

		if( (nz != (nval-3)/2) || (nz < 3) ) {
			QString messg;
			messg.sprintf("\"%s\" Row %d: Invalid format?",
				fileName.data(), nRow);
			printf("%s\n",messg.data());
			exit(1);
//			throw LoadError(messg);
		}

		ut[n_ut] = ut_;
		n_ut++;

//		printf("(%d,%d)->%d::%d::%d(%5.2f) - %f %e   %f %e   %f %e\n",
//			nval,nz,h,m,s,ut[n_ut],z[n_ut][0],cn2[n_ut][0],
//			z[n_ut][1],cn2[n_ut][1],z[n_ut][2],cn2[n_ut][2]);
	}
/*
	for(int i=0; i<n_ut; i++) {
		printf("%f ",ut[i]);
		for(int j=0; j<nz; j++) {
			printf("%7.3f %e   ",z[i][j],cn2[i][j]);
		}
		printf("\n");
	}
*/
/*
	delete[] ut;
	for(int i=0; i<max_n_ut; i++) {
		delete[] z[i];
		delete[] cn2[i];
	}
	delete[] z;
	delete[] cn2;
*/
	double sqrt2 = sqrt(2);
	nZsmooth = int(ceil(logN(sqrt2,maxAlt))+2);
	zSmooth = new double[nZsmooth];
	cn2Smooth = new double[nZsmooth];
	zSmooth[0] = sqrt2/2;
	for(int i=1; i<nZsmooth; i++) {
		zSmooth[i] = zSmooth[i-1] * sqrt2;
//		debug("zSmooth=%5.2f",zSmooth[i]);
	}

	return 1;
}


/**
	@brief	Compute the ticks origin/spacing and axis limits for graphics
	@param	min	On enter: data minimum; on exit: axis origin (\a min - lmargin)
	@param	max	On enter: data maximum; on exit: axis end (\a max + rmargin)
	@param	expand	range expansion factor (relative "margins" size)
	@param	n	desired ("optimal") number of ticks
	@param	tick0	resulting coordinate (numeric value) of a first tick
	@param	dtick	resulting tick spacing
	@return	Number of resulted tick marks (in range ~ \a n/1.5..n*1.5)
	
	The utility acceps the range \a min..max of data for a graphic display
	(either abscissa or ordinate, no matters) and  adjusts the axis limits and
	tick marks origin and spacing to have \e about \a n ticks per axis. The
	ticks label values (i.e. coordinates) are adjusted to be "rounded", i.e. to
	contain the least possible number of significant digits.

	The axis limits are expanded with respect to the data range \a min..max to
	have some "margins" each of the size not less than \a expand*(max-min).
	Also, if only "a little" is needed in expansion to include a new tick mark
	at the exact axis limit, the latter is adjusted correspondingly.
	
	\e Example:
	
	Given the range \a min=22.01, \a max=22.87, \a expand=0.050, the
	utility returns:
	
	\a min = 21.967, \a max = 22.913 (recommended axis limit), \a tick0 = 22, 
	\a dtick = 0.1, returned value (number of ticks) is 10.
	
	This means that the ticks are to be placed at:
	
	22 22.1 22.2 22.3 22.4 22.5 22.6 22.7 22.8 22.9 .
*/
int graphticks(double& min, double& max, double expand, int n,
	double& tick0, double& dtick) {
	const double crit = 0.8;	// maximal distance of a marginal tick from 
						// a range limit in [dtick]
	if (min>=max) return 0;

	// expansion size: 
	double esize = expand * (max-min);

	// null-approximation of dtick:
	double d0 = (max-min+2*esize)/(n-1);
	// ... and its order:
	double order = floor(log10(d0));
	// convert dtick to the range 1..10 and take the nearest integer value:
	double d1 = rint(d0*pow(10.,-order));
	// change to the nearest "standard" ticks distance:
	if (d1>7)
		d1 = 10;
	else
		if (d1<8 && d1>3)
			d1 = 5;
		else
			if (d1==3)
				d1=2.5;

	// the number of resulted ticks:
	int N1 = int((max-min+2*esize)/d1/pow(10.,order)) ;
	// double or half it if out of range:
	const double sqrt2 = sqrt(2.);
	if (N1<n/sqrt2) {
		if (d1==2.5)
			d1 = 1;
		else
			d1 /= 2 ;
	} else 
		if (N1>n*sqrt2) {
			if (d1==2)
				d1 = 5;
			else
				d1 *= 2;
		}

	// convert back to the original scale:		
	dtick = d1 * pow(10.,order);

	// First tick position accounting the range expansion:
	tick0 = int(ceil((min-esize)/dtick))*dtick;
	// expand more than by esize if the first tick is "too far" from min:
	if ((tick0-min)/dtick > crit)
		// expand a little more to include a new tick:
		min = tick0 = tick0-dtick;
	else
		min -= esize;	// expand as ordered by "expand"

	// number of ticks now accounting the range expansion:
	N1 = int((max+esize-tick0)/dtick)+1 ;

	// last tick position accounting the range expansion:
	double tickl = tick0 + (N1-1)*dtick;
	// expand again more than by esize if the last tick is "too far" from max:
	if ((max-tickl)/dtick > crit) {
		// expand a little more to include a new tick:
		max = tickl = tickl+dtick;		
		N1 ++ ;
	} else
		max += esize;	// expand as ordered by "expand"

	return N1 ;
}

void GraphSet::load(QString& pname_,
			const char* color_,
			int size_,
			double refer,
			double* x,
			double* y,
			int nP,
			double scale) {
	_color = color_;
	_size = size_;
	_refer = refer;
	_xData = x;
	_yData = y;
	_nPoint = nP;
}

GraphWindow::GraphWindow(QWidget* parent,const char* name)
			: QWidget(parent,name) {


	char* fname = "";
	double refer = 3.0e-13;
	char* lutname = "/usr/share/turbina/lut/heat.lut";
	_background = darkGreen;
	_axesColor = white;
	_fontSize = 15;
	_t1 = -1;
	_t2 = -1;

	int i;
	while( (i = getopt(qApp->argc(),qApp->argv(),"m:r:l:b:a:f:s:e:h")) != -1) {
		switch(i) {
			case 'h' : fprintf(stdout,"Usage: turgraph [-hmrlbafse] [mass-file]\n\
   -h    this help\n   -m    method(L)\n   -r    refer. value(3e-13)\n   -l    lut(heat)\n\
   -b    backround(gray)\n   -a    axes(yellow)\n   -f    fontsize(15)\n\
   -s    starttime\n   -e    endtime\n"); exit(1);
			case 'm' : if(optarg) met=*optarg; break;
			case 'r' : if(optarg) sscanf(optarg,"%lf",&refer); break;
			case 'l' : if(optarg) lutname=optarg; break;
			case 'b' : if(optarg) _background=optarg; break;
			case 'a' : if(optarg) _axesColor=optarg; break;
			case 'f' : if(optarg) sscanf(optarg,"%d",&_fontSize); break;
			case 's' : if(optarg) sscanf(optarg,"%lf",&_t1); break;
			case 'e' : if(optarg) sscanf(optarg,"%lf",&_t2); break;
//			default	 : exit(1);
		}
	}
	if(qApp->argv()[optind] != NULL) fname=qApp->argv()[optind];
	else {
		fprintf(stdout,"Usage: turgraph [-options] [mass-file]\n");
		exit(1);
	}

	_minLutVal = 0;
	_maxLutVal = 0;
	_font = QFont("Courier",_fontSize);
	QString lutFileName(lutname);
	loadLutTable(lutFileName);
	if( loadProfile(fname, _t1, _t2) != 1) {
		fprintf(stdout,"Invalid format...:-(\n");
		exit(1);
	}
	

	setCaption("CN2");

	_winX = qApp->desktop()->width()/6;	
	_winY = 0;
	_winWidth = qApp->desktop()->width()/3*2 - 10;
	_winHeight = qApp->desktop()->height()/2 - 50;
	setGeometry(_winX+10, _winY+68, _winWidth, _winHeight);
	//      Pochemu +10 i +68 NE ZNAYU !!!!!
	setBackgroundColor(_background);
		QString pname("CN2");
//		_gs.load(pname, "green",1,6.0e-13, NULL,NULL,n_ut,1);
		_gs.load(pname, "green",1,refer, NULL,NULL,n_ut,1);
}

GraphWindow::~GraphWindow(void) {
	if(zSmooth) delete[] zSmooth;
	if(cn2Smooth) delete[] cn2Smooth;
	if(ut) delete[] ut;
	for(int i=0; i<max_n_ut; i++) {
		delete[] z[i];
		delete[] cn2[i];
	}
	if(z) delete[] z;
	if(cn2) delete[] cn2;
//	killTimer(timer);
}

int toInt255(double val) {
	val *= 255;
	if(val < 0) val = 0;
	else if(val > 255) val = 255;
	return int(val);
}

void GraphWindow::loadLutTable(const QString& fileName) {
	QFile file(fileName);
	if(!file.open(IO_ReadOnly)) {
		QString messg = "\"" + fileName + "\" not found!";
		printf("%s\n",messg.data());
		exit(1);
//		throw LoadError(messg);
	}
	QTextStream textStream(&file);
	int nColor = 0;
	int nRow = 0;

	while(!textStream.eof()
			&& (nColor < LUT_LENGTH)) {
		QString line = textStream.readLine();
		nRow++;

		if(line.isEmpty()) continue;
		if(line[0] == '#') continue;

		int n;
		double r,g,b;
		int nval = sscanf(line.data(), "%d%lf%lf%lf", &n, &r,&g,&b);
		if(nval != 4) {
			QString messg;
			messg.sprintf("\"%s\" Row %d: Invalid format!",
				fileName.data(), nRow);
			printf("%s\n",messg.data());
			exit(1);
//			throw LoadError(messg);
		}
		lutTable[nColor].setRgb(toInt255(r),toInt255(g),toInt255(b));
		nColor++;
	}
	if(nColor != LUT_LENGTH) {
		QString messg = fileName + " is corrupted!";
		printf("%s\n",messg.data());
		exit(1);
//		throw LoadError(messg);
	}
	
}

const QColor& GraphWindow::getLutColor(double val) {
	if(val < 0) val = 0;
	else if(val > LUT_LENGTH-1) val = LUT_LENGTH-1;
	return lutTable[int(val)];
}

void GraphWindow::setLutColor(QPainter* p, double val, double maxVal) {
	double lutVal = 255;
	if(maxVal != 0) lutVal = val / maxVal * 255;
	QColor lutColor = getLutColor(lutVal);
	p->setPen(lutColor);
	p->setBrush(lutColor);
}

void GraphWindow::drawLut(QPainter *bp,
			int left,
			int top,
			int w, //width
			int h, //height
			double minVal,
			double maxVal) {


	bp->setViewport(left, top, w, h);
	int wt = 1000;
	int wb = 0;
	int wr = 1000;
	bp->setWindow(0,wt,wr+1,wb-1-wt);

	int port_top = top;
	int port_height = h+_border_y;
	bp->setViewXForm(false);
	bp->fillRect(0,
		port_top,
		width(),
		port_height,
		_background);
	bp->setViewXForm(true);

	const int nStep = 250;
	const int step_width = 4;
	int step_left = 0;
	int step_height = wt / 2;


	for(int i=0; i<nStep; i++) {
		setLutColor(bp, double(i), double(nStep-1));
		bp->drawRect(step_left, step_height,
					step_width+1, step_height);
		step_left += step_width;
	}

	bp->setPen(_axesColor);
	int th = bp->fontMetrics().height();
	QString txt;

	bp->setViewXForm(true);
	QPoint b = bp->xForm(QPoint(wr,step_height));
	bp->setViewXForm(false);
	txt.setNum(maxVal);
	int tw = bp->fontMetrics().width(txt);
	bp->drawText(b.x()-tw,b.y()+th,txt);

	bp->setViewXForm(true);
	b = bp->xForm(QPoint(0,step_height));
	bp->setViewXForm(false);
	txt.setNum(minVal);
	tw = 3 * bp->fontMetrics().width(txt);
	bp->drawText(b.x(),b.y()+th,txt);
}

const double T_FACT = 1.5;

bool GraphSet::searchMinMax(void) {
	if(_nPoint == 0) {
		_xMin = 0;
		_xMax = 0;
		_yMin = 0;
		_yMax = 0;
		return true;	
	}
// Y:
	_yMin = 0;
	_yMax = 0;
	_yMin = zSmooth[0];
	_yMax = zSmooth[nZsmooth-1];
// X:
	_xMin = ut[0];
	_xMax = ut[_nPoint-1];
//	_xMax = _xMin + 2.0;	// init: 2h
//	while(ut[_nPoint-1] > _xMax) {
//		_xMax = (_xMax-_xMin) * T_FACT + _xMin;
//		debug("ut=%f xmax=%f\n",ut[_nPoint-1], _xMax);
//	}
	return 1;
}

int wDevX(double x, Scale* s) {
	return int(rint((x-s->xmin)*s->sx));
}

int wDevY(double y, Scale* s) {
	return int(rint((y-s->ymin)*s->sy));
}

QPoint wDev(double x, double y, Scale* s) {
	return QPoint(wDevX(x,s),wDevY(y,s));
}

double devWX(int x, Scale* s) {
	return((double)x / s->sx + s->xmin);
}

double devWY(int y, Scale* s) {
	return((double)y / s->sy + s->ymin);
}

const double minFform = 0.0001;
const double maxFform = 10000;
const int G_FORM = -1;

int getPrec(double val) {

	double val_abs = fabs(val);
	if((val_abs<minFform)
		|| (val_abs>=maxFform)) return G_FORM; // g-format
	int prec = 0;
	if(val_abs < 1.0) prec = 3;
	else if(val_abs < 10.0) prec = 2;
	else if(val_abs < 100.0) prec = 1;
	else prec = 0;

	return prec;
}

void toStr(QString& txt, double val, int prec) {

	char form = 'f';
	if(prec == G_FORM) form = 'g';
	txt.setNum(val,form,prec);
}

void GraphWindow::drawX(QPainter *p,
			Scale *s,
			int optimNTicks,
			bool top,
			const char* name) {
	QPen oldpen = p->pen();
	p->setPen(_axesColor);

	int wl = p->window().left();
	int wr = p->window().right();
	int wt = p->window().top();
	int wb = p->window().bottom();

	int tw = p->fontMetrics().width("wwwwwww")+10; 
	int th = p->fontMetrics().height();
	QPoint p1 = p->xFormDev(QPoint(0,0)); 
	QPoint p2 = p->xFormDev(QPoint(0,
		int(rint(0+0.7*th)))); 
	int swy = abs(p1.y() - p2.y());

	if(top) p->drawLine(wl,wt,wr,wt);
	else p->drawLine(wl,wb,wr,wb);

	double tick0=0, dtick=0;
	double min = s->xmin;
	double max = s->xmax;

	int nTicks = graphticks(min,max,0,
				optimNTicks,tick0,dtick);

	int prec = getPrec(tick0 + dtick*(nTicks-1));
	double tick = tick0;
	for(int i=0; i<nTicks; i++) {
		int ix = wDevX(tick,s);
		if((ix < wl) || (ix > wr)) {
			tick += dtick;
			continue;
		}
		p->setViewXForm(true);
		QPoint b;
		if(top) {
			p->drawLine(ix,wt,ix,wt-swy);
			b = p->xForm(QPoint(ix,wt));
		}else {
			p->drawLine(ix,wb,ix,wb+swy);
			b = p->xForm(QPoint(ix,wb));
		}
		p->setViewXForm(false);
		QString txt; 
		toStr(txt, tick, prec);
		tw = p->fontMetrics().width(txt);
		if(top) p->drawText(b.x()-tw/2,b.y()-th/2,txt);
		else p->drawText(b.x()-tw/2,b.y()+th,txt);
		tick += dtick;
	}

// Draw Name:
	p->setViewXForm(true);
	QPoint b;
	if(top) b = p->xForm(QPoint(wr,wt));
	else b = p->xForm(QPoint(wr,wb));
	p->setViewXForm(false);
	tw = 3 * p->fontMetrics().width(name);
	if(top) p->drawText(b.x()-tw/2,
					int(rint(b.y()-1.5*th)), name);
	else p->drawText(b.x()-tw/2,b.y()+2*th,name);

	p->setViewXForm(true);
	p->setPen(oldpen);
}

void GraphWindow::drawX(QPainter *p) {

	QPen oldpen = p->pen();
	p->setPen(_axesColor);

	int wl = p->window().left();
	int wr = p->window().right();
	int wb = p->window().bottom();

	p->drawLine(wl,wb,wr,wb);
	p->setPen(oldpen);
}

void GraphWindow::drawY(QPainter *p, Scale *s,
			int optimNTicks,
			bool left,
			bool drawNull,
			const char* name,
			bool logY) {
	QPen oldpen = p->pen();
	p->setPen(_axesColor);

	int wl = p->window().left();
	int wr = p->window().right();
	int wt = p->window().top();
	int wb = p->window().bottom();

	int tw = p->fontMetrics().width("wwwwwww")+10; 
	int th = p->fontMetrics().height();
	int th2 = th/3;

	QPoint p1 = p->xFormDev(QPoint(0,0)); 
	QPoint p2 = p->xFormDev(QPoint(0+int(rint(0.7*th)),0)); 
	int swx = abs(p1.x() - p2.x());

	p->setViewXForm(true);
	if(left) p->drawLine(wl,wt,wl,wb);
	else p->drawLine(wr,wt,wr,wb);

	if(drawNull) {
		int x0 = wDevX(0,s);
		p->drawLine(x0, 0, x0, wt);
	}

	double tick0=0, dtick=0;
	double min = s->ymin;
	double max = s->ymax;

	int nTicks = graphticks(min,max,0,
			optimNTicks,tick0,dtick);

	int prec = getPrec(tick0 + dtick*(nTicks-1));
	double tick = tick0;
	for(int i=0; i<nTicks; i++) {
		int iy = wDevY(tick,s);
		if((iy < wb) || (iy > wt)) {
			tick += dtick;
			continue;
		}
		p->setViewXForm(true);
		if(left) p->drawLine(wl,iy,wl+swx,iy);
		else p->drawLine(wr,iy,wr-swx,iy);
		QPoint b = p->xForm(QPoint(wl,iy));
		p->setViewXForm(false);
		QString txt;
		if(left) {
			if(logY)
				toStr(txt, fromLogScale(tick), prec);
			else toStr(txt, tick, prec);
			tw = p->fontMetrics().width(txt);
			p->drawText(b.x()-tw,b.y()+th2,txt);
		}
		tick += dtick;
	}
	if(left) {
		p->setViewXForm(true);
		QPoint b = p->xForm(QPoint(wl,wt));
		p->setViewXForm(false);
//		tw = p->fontMetrics().width("  ");
		tw = p->fontMetrics().width(name) * 2;
		p->drawText(b.x()-tw,b.y()+int(rint(1.3*th)),name);
		p->drawText(b.x()-tw/5*2,b.y()+int(rint(1.3*th)),&met);
	}
	p->setViewXForm(true);
	p->setPen(oldpen);
}

void GraphWindow::drawViewPort(	GraphSet* gs,
			QPainter *bp,
			int left,
			int top,
			int w, // width
			int h, // height
			bool drawTop,
			bool drawBottom) {

	if(!gs) return;
	bp->setViewport(left, top, w, h);

	int optimNTicksX = w / (2*bp->fontMetrics().width("ww.w")); 
	int optimNTicksY = h / (2*bp->fontMetrics().height());
	if(optimNTicksY < 2) optimNTicksY = 2;

	int wr = 1000;
	int wt = 1000;

	gs->searchMinMax();
	int port_top = top-_border_y/2;
	int port_height = h+_border_y;//+_border_y/2;
	if(drawTop) {
		port_top -= _view_marg;
		port_height += _view_marg;
	}
	if(drawBottom) {
		port_height += _view_marg+_border_y;
	}
	bp->setViewXForm(false);
	bp->fillRect(0,
		port_top,
		width(),
		port_height,
		_background);
	bp->setViewXForm(true);
	Scale s(gs->xMin(), gs->xMax(), gs->yMin(), gs->yMax());
	s.ymin = toLogScale(s.ymin) - 0.5;
	s.ymax = toLogScale(s.ymax) + 0.5;

	if(s.xmax == s.xmin) {
		s.xmax++;
		s.xmin--;
	}
	if(s.ymax == s.ymin) {
		s.ymax++;
		s.ymin--;
	}

	double tick0=0, dtick=0;
	const double expand = 0.05;
	graphticks(s.xmin,s.xmax,expand,optimNTicksX,tick0,dtick);
	graphticks(s.ymin,s.ymax,0,optimNTicksY,tick0,dtick);

	s.sx = wr / (s.xmax - s.xmin); 
	s.sy = wt / (s.ymax - s.ymin);

	int dh = bp->fontMetrics().height();
	bp->setClipRect(left,top-dh/2,w,h+dh);

	bp->setClipping(true);
	drawCurve(gs, bp, &s, wt, wr);
	bp->setClipping(false);

	const char* nameX;
	nameX = "UT,h";
	if(drawTop) drawX(bp,&s,optimNTicksX,true,nameX);
	if(drawBottom) drawX(bp,&s,optimNTicksX,false,nameX);
	else drawX(bp);
	drawY(bp,&s,optimNTicksY,true,false,
			"CN2",true);
	drawY(bp,&s,optimNTicksY,false,false,
			"",true);
}



void GraphWindow::drawAll(void) {
	QPainter bufferpainter;
	bufferpainter.begin(&_buffer);
	_buffer.fill(_background);
	bufferpainter.setFont(_font);
	const int border_x = 20;
	_border_y = bufferpainter.fontMetrics().height();
	int view_left = bufferpainter.fontMetrics().width("wwwwwww")+10;
	_view_marg = int((double)bufferpainter.fontMetrics()
			.height()*2.5);
	int view_top = _view_marg;
	int lut_height = 0;
	lut_height = _border_y;
	int view_height = height() - 2*_view_marg - lut_height -
//			_border_y*(outcontrol.nGraph()-1) -
			_border_y;
	int view_width = width()-view_left-border_x;

	bool x_top = true; // x-ax.
	bool x_bottom = true; // x-axis

//		GraphSet gs;
	drawViewPort(&_gs,
		&bufferpainter,
		view_left,
		view_top,
		view_width,
		view_height,
		x_top,
		x_bottom);

		view_top += int((double)view_height*1
			 + _border_y);

	view_top += _view_marg - _border_y;
	drawLut(&bufferpainter,
			view_left,
			view_top,
			view_width,
			lut_height,
			_minLutVal,
			_maxLutVal);
	bufferpainter.end();
 	repaint();
}

void GraphWindow::drawCurve(GraphSet* gs,
				QPainter* bp,
				Scale* s,
				int wt,
				int wr) {
	bp->setWindow(0,wt,wr+1,-1-wt);
	if(!gs) return;
	if(!gs->nPoint()) return;

	QColor color(gs->color());
	bp->setBrush(color);
	bp->setPen(color);

	QPoint es1 = bp->xFormDev(QPoint(10,10)); 
	QPoint es2 = bp->xFormDev(QPoint(10+_pointSize,
					10+_pointSize)); 
	int ax = abs(es1.x() - es2.x());
	int ay = abs(es1.y() - es2.y());

	if(ay == 0) ax = 1;


	int startPointNumber = 0;
	double value = gs->refer();
	if(value != _maxLutVal) {
		_maxLutVal = value;
	}
	_minLutVal = 0;
	int startProfile = startPointNumber > 0 ?
				startPointNumber-1 : 0;
	for(int i=startProfile; i<gs->nPoint(); i++) {
		int nCN2 = nz;	// !!! poka!!!
		if(nCN2 == 0) continue;

		double* zCN2_ = z[i];
		double* cn2_ = cn2[i];

		atm::interpolate(zSmooth,
				cn2Smooth,
				nZsmooth,
				zCN2_, cn2_, nCN2);
		cn2_ = cn2Smooth;
		zCN2_ = zSmooth;
		nCN2 = nZsmooth;

		int ixl = i, ixr = i;
		if(i > 0) ixl = i-1;
		if(i < gs->nPoint()-1) ixr = i+1;

		const double maxTimeInterval = 1.0; // 1 hour
		double xDataL = ut[i];
		if( fabs(ut[ixl]-ut[i]) < maxTimeInterval) xDataL = ut[ixl];
		double xDataR = ut[i];
		if(fabs(ut[ixr]-ut[i]) < maxTimeInterval) xDataR = ut[ixr];

		int axL = abs(wDevX(ut[i],s)
				 - wDevX(xDataL,s)) / 2;
		int axR = abs(wDevX(ut[i],s)
				 - wDevX(xDataR,s)) / 2;
		ay = int(wt / nCN2 / 2);

		for(int j=0; j<nCN2; j++) {
			setLutColor(bp, cn2_[j], gs->refer());
			int x = wDevX(ut[i],s) - axL;
			int y = wDevY(toLogScale(zCN2_[j]),s) - ay;
			int w = axL+axR+2;
			int h = 2*ay;
			bp->drawRect(x,y,w,h);
		}
	}
}

void GraphWindow::paintEvent(QPaintEvent* e) {
	bitBlt(this, 0, 0, &_buffer);
}

void GraphWindow::resizeEvent(QResizeEvent* e) {
	_buffer.resize(e->size());
	_buffer.fill(_background);
	drawAll();
}

