/*--------------------------------------------------------------------------*/
/** @file graph.h
@brief  Supports the output of the measurement results in the graphic window.

                  MASS project: TURBINA module file
                       header file for graph.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef GRAPH_H
#define GRAPH_H

#define LUT_LENGTH      256

#include <qpainter.h>
#include <qpixmap.h>
#include <qwidget.h>

class GraphSet {
private:
//public:
	QString		_pname;
	double*		_xData;
	double*		_yData;
	double		_refer;
	double		_xMin;
	double		_xMax;
	double		_yMin;
	double		_yMax;
	int		_nPoint;
	int		_nPointGraph;
	int		_size;
	double		_quantum; // 0.0 .. 1.0
	const char*	_color;
	bool		_changed;

public:

	GraphSet( void ) { _changed = true; _xData = NULL;
		_yData = NULL; _nPoint = 0; 
		_nPointGraph = 0; _refer = 0; }
	void load( QString& pname_,
			const char* color,
			int size_,
			double refer,
			double* x = NULL,
			double* y = NULL,
			int nP = 0,
			double scale = 1 );
/*
	void		setData( double* x,
					double* y,
					int nP );
	void		setNPoint( int nP );
	void		setNPointGraph( int nP );
	void		setQuantum( double quant ) {_quantum = quant;}
	void		setChanged( bool chng ) { _changed = chng; }
	bool		isChanged( void ) { return _changed; }
	double*		xData( void ) { return _xData; }
	double*		yData( void ) { return _yData; }
*/
	double		refer( void ) { return _refer; }
/*
	void		setRefer( double refer ) { _refer = refer; }
*/
	int		nPoint( void ) {return _nPoint; }
/*
	int		nPointGraph( void ) {return _nPointGraph; }
*/
	bool		searchMinMax( void );

	double		xMin( void ) { return _xMin; }
	double		xMax( void ) { return _xMax; }
	double		yMin( void ) { return _yMin; }
	double		yMax( void ) { return _yMax; }
	const char*	color( void ) {return _color;}
/*
	double		quantum( void ) {return _quantum;}

	int		size( void ) {return _size;}
	const char*	pname( void ) {return _pname.data(); }
	bool		isScan( void );
	bool		isProfile( void );
*/
};



struct Scale {
	Scale::Scale(double xMn, double xMx,
				double yMn, double yMx) {
				xmin = xMn;
				xmax = xMx;
				ymin = yMn;
				ymax = yMx;
			}
	double xmin;
	double xmax;
	double ymin;
	double ymax;
	double sx;
	double sy;
};

class GraphWindow : public QWidget {
//	Q_OBJECT
private:
	QColor          lutTable[LUT_LENGTH];

	QPixmap _buffer;

	bool	empty;

	GraphSet _gs;

	int	_winX;
	int	_winY;
	int	_winWidth;
	int	_winHeight;
	int	_border_y;
	int	_view_marg;
	QFont	_font;
	int	_fontSize;
	int	_pointSize;
	QColor	_background;
	QColor	_axesColor;
	double	_t1;	//     (c)
	double	_t2;	// 

	int	timer;
	bool	isTimerEventLocked;

	double	_maxLutVal;
	double	_minLutVal;
public:
	GraphWindow(QWidget *parent=0, const char *name=0);
	~GraphWindow(void);
	void drawAll(void);

private:
	void GraphWindow::loadLutTable(const QString& fileName);
	const QColor& getLutColor(double val);
//	void loadConst(bool first=true);

	void drawX(QPainter*,
			Scale*,
			int,
			bool top=false,
			const char* name="");
	void drawX(QPainter*);
	void drawY(QPainter*,
			Scale*,
			int,
			bool left,
			bool drawNull=false,
			const char* name="",
			bool logY=false);


	void drawViewPort(GraphSet* gs,
				QPainter* bp,
				int left,
				int top,
				int width,
				int height,
				bool drawTop=false,
				bool drawDawn=false);

	void setLutColor(QPainter* p, double val, double maxVal);

	void drawCurve(GraphSet* gs,
			QPainter* bp,
			Scale* s,
			int wt,
			int wr);

	void drawLut(QPainter* bp,
			int left,
			int top,
			int width,
			int height,
			double minVal,
			double maxVal);


/*
private slots:
	void change(void);

*/
protected:
//	void timerEvent(QTimerEvent*);
	virtual void paintEvent(QPaintEvent*);
	virtual void resizeEvent(QResizeEvent*);
//	virtual void moveEvent(QMoveEvent*);
//	virtual void mousePressEvent(QMouseEvent*);
//	virtual void mouseMoveEvent(QMouseEvent*);

};

#endif
