/**     @file unroll.h
@brief	Converts the scenario formula in the sequence of mode symbols.


	MASS project: header file for unroll.cpp
 
	The module implements the text parser for "unrolling" the MASS scenario
	records. The record is the string where the different modes follow
	after each other as "A+B" and different sequences of modes of an
	arbitrary length are repeated a certain amount of times as "20*(A+B)".
	The idea of implementation of this module comes from the "calculator"
	program by Bjarne Stroustrup (see "Jazyk Programmirovanija C++", 3rd
	edition, Moscow, Binom, 2001, p. 147).
*/
#ifndef _UNROLL_H
#define _UNROLL_H

#include <string>

/**
	@brief	Unroll the scenario record in a string of mode symbols
	@param	rec 	scenario record
	@param 	dict	"dictionary": a list of allowed mode symbols to reject
		the erroneous input. Case sensitive.
	@return	string which is an exact sequence in which the planned modes will be
		started
	
	The allowed operators are 
	- "+" addition for adding the modes or the sequences of modes. Addition of
	numbers or numbers and modes is not allowed;
	- "*" multiplication - for repeating the sequences or modes many times.
	Defined only for integer numbers or modes/sequences and integer numbers, in
	 arbitrary order.
	- "(...)" for grouping the modes in sequences, with arbitrary degree of
	enclosure.
	
	Only the integer numbers are allowed for multiplication. Spaces are allowed
	in a record.
	
	Example:
	\verbatim
		string scenario = unroll("2*(A+3*B+C)","ABC") ;
	\endverbatim
		returns "ABBBCABBBC".
*/
const std::string& unroll(const char * rec, const char * dict) ;

#endif
